/*****************************************************************************
 * RdfGraph.cs
 * 
 * Copyright (c) 2002, Rahul Singh.
 *  
 * This file is part of the Drive RDF Parser.
 * The Drive RDF Parser is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * The Drive RDF Parser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with the Drive RDF Parser; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Author: 
 * 
 * Rahul Singh
 * kingtiny@DriveRDF.org
 * 
 ******************************************************************************/

using System;
using System.Collections;

namespace Drive.Rdf
{
	/// <summary>
	/// Represents an RDF Graph. 
	/// </summary>
	/// <remarks>This is an implementation of the IRdfGraph interface. This class maintain a collection of Nodes and a separate collection of Litrerals
	/// in order to allow distinguishing between Nodes and Literals with the same value.</remarks>
	public class RdfGraph : IRdfGraph
	{
		/// <summary>
		/// The namespaces associated with this RDF Graph
		/// </summary>
		private RdfNamespaceCollection _nameSpaces;

		/// <summary>
		/// Gets the namespaces associated with this RDF Graph
		/// </summary>
		public IRdfNamespaceCollection NameSpaces
		{
			get
			{
				return _nameSpaces;
			}
		}

		/// <summary>
		/// The collection of RdfNodes in this Graph
		/// </summary>
		private RdfNodeCollection _nodes;

		/// <summary>
		/// Gets the collection of RdfNodes in this Graph
		/// </summary>
		public IRdfNodeCollection Nodes
		{
			get
			{
				return (IRdfNodeCollection)_nodes;
			}
		}

		/// <summary>
		/// The collection of literals in this RDF Graph
		/// </summary>
		private RdfNodeCollection _literals;

		/// <summary>
		/// Gets the collection of literals in this RDF Graph
		/// </summary>
		public IRdfNodeCollection  Literals
		{
			get
			{
				return (IRdfNodeCollection)_literals;
			}
		}

		/// <summary>
		/// Gets the total number of nodes in this RDF Graph
		/// </summary>
		public long Count
		{
			get
			{
				return _nodes.Count+_literals.Count;
			}
		}

		/// <summary>
		/// Adds a node to the Graph
		/// </summary>
		/// <param name="nodeUri">A string representing the URI of the new node.</param>
		/// <exception cref="UriFormatException">The specified nodeUri is not a well formed URI.</exception>
		/// <returns>An object that implements the IRdfNode interface. This is a reference to the new node added.
		/// This method checks the graph to determine whether the node with the specified URI exists. 
		/// If it does then a reference to the existing node is returned. If it does not exist then a new node is created, added 
		/// to the graph and returned.</returns>
		public IRdfNode AddNode(string nodeUri)
		{
			IRdfNode node = _nodes[nodeUri];
			if(node == null)
			{
				node = new RdfNode(nodeUri);
				_nodes[nodeUri] = node;
			}
			return node;
		}

		/// <summary>
		/// Adds a new node to the graph.
		/// </summary>
		/// <param name="node">An object that implements the IRdfNode interface. This is the new node to add.</param>
		/// <exception cref="ArgumentException">A node with the same ID already exists in the Graph.</exception>
		public void AddNode(IRdfNode node)
		{
			_nodes.Add((RdfNode)node);
		}

		/// <summary>
		/// Adds a literal to the Graph
		/// </summary>
		/// <param name="literalValue">A string representing the value of the literal.</param>
		/// <returns>An object that implements the IRdfLiteral interface. This is a reference to the newly added literal.</returns>
		/// <remarks>This method looks in the graph to determine whether a literal with the specified value (and a null datatype and langID) exists
		/// in the Graph. If the literal exists a reference to the existing literal is returned. If it does not exist then a new literal (with the specified value, and null datatype and LangID)
		/// is created, added to the graph and returned.</remarks>
		public IRdfLiteral AddLiteral(string literalValue)
		{
			RdfLiteral literal = (RdfLiteral)_literals[literalValue];
			if(literal == null)
			{
				literal = new RdfLiteral(literalValue);
				_literals[literalValue] = literal;
			}
			return literal;
		}

		/// <summary>
		/// Adds a literal to the Graph
		/// </summary>
		/// <param name="literalValue">A string representing the value of the literal.</param>
		/// <param name="langID">A string representing the Language ID of the literal.</param>
		/// <param name="datatypeUri">A string representing the datatype URI of the literal.</param>
		/// <returns>An object that implements the IRdfLiteral interface. This is a reference to the newly added literal.</returns>
		/// <exception cref="UriFormatException">The specified datatype URI is not null and is not a well formed URI.</exception>
		/// <remarks>This method looks in the graph to determine whether a literal with the specified value, datatype and langID exists
		/// in the Graph. If the literal exists a reference to the existing literal is returned. If it does not exist then a new literal with the specified value, datatype and LangID
		/// is created, added to the graph and returned. Any parameter supplied to this method can be null or empty and it will be ignored.</remarks>
		public IRdfLiteral AddLiteral(string literalValue, string langID, string datatypeUri)
		{
			string literalID = literalValue;
			if((langID != null) && (langID.Length != 0))
				literalID +="@"+langID;
			if((datatypeUri != null) && (datatypeUri.Length != 0))
				literalID +="^^"+datatypeUri;
			RdfLiteral literal = (RdfLiteral)_literals[literalID];
			if(literal == null)
			{
				literal = new RdfLiteral(literalValue,langID,datatypeUri);
				_literals[literalID] = literal;
			}
			return literal;
		}

		/// <summary>
		/// Adds a literal to the graph
		/// </summary>
		/// <param name="literal">The new literal to add.</param>
		/// <exception cref="ArgumentException">A literal with the same value, datatype URI and language ID alreday exists in the graph.</exception>
		public void AddLiteral(IRdfLiteral literal)
		{
			_literals.Add((RdfLiteral)literal);
		}

		/// <summary>
		/// Gets the node (or literal) with the specified URI
		/// </summary>
		/// <remarks>This method looks for a node that matches the specified URI and returns it. 
		/// If the node is not found then the first literal matching this URI (value+langiuageID+datatype URI) is returned.
		/// If neither a node or a literal matching this ID is found then null is returned.</remarks>
		public IRdfNode this[string nodeID]
		{
			get
			{
				IRdfNode node = _nodes[nodeID];
				if(node == null)
					node = _literals[nodeID];
				return node;
			}
		}

		/// <summary>
		/// Initializes a new instance of the RdfGraph class.
		/// </summary>
		public RdfGraph()
		{
			_nodes = new RdfNodeCollection();
			_literals = new RdfNodeCollection();
			_nameSpaces = new RdfNamespaceCollection();
		}

	}
}

