/*****************************************************************************
 * RdfParser.cs
 * 
 * Copyright (c) 2002, Rahul Singh.
 *  
 * This file is part of the Drive RDF Parser.
 * The Drive RDF Parser is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * The Drive RDF Parser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with the Drive RDF Parser; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Author: 
 * 
 * Rahul Singh
 * kingtiny@DriveRDF.org
 ******************************************************************************/

using System;
using System.Xml;
using System.Collections;

namespace Drive.Rdf
{
	/// <summary>
	/// The primary RDF Parser.
	/// </summary>
	public class RdfParser : IRdfParser
	{
		/// <summary>
		/// A Collection of Properties in the RDF syntax
		/// </summary>
		/// <remarks>The members of this collection are rdf:resource, rdf:parseType, rdf:ID, rdf:nodeID, rdf:bagID, rdf:datatype, rdf:about</remarks>
		private Hashtable _rdfProperties;

		/// <summary>
		/// A Collection of all RDF and XML properties
		/// </summary>
		/// <remarks>The members of this collection are rdf:about, rdf:resource, rdf:parseType, rdf:ID, rdf:nodeID, rdf:bagID, rdf:datatype, xml:lang</remarks>
		private Hashtable _rdfXmlProperties;

		/// <summary>
		/// Variable used to generate new IDs for RDF Nodes
		/// </summary>
		private long _newID;

		/// <summary>
		/// Total number of errors encountered while parsing the RDF/XML
		/// </summary>
		private long _errorCount;

		/// <summary>
		/// Gets the total number of errors encountered while parsing the RDF/XML
		/// </summary>
		public long Errors
		{
			get
			{
				return _errorCount;
			}
		}

		/// <summary>
		/// Indicates whether exceptions should be thrown when an error is encountered.
		/// </summary>
		private bool _exceptionsOnError;

		/// <summary>
		/// Gets or sets whether Exceptions should be thrown when an error is encountered.
		/// </summary>
		/// <remarks> The default value is true. Setting this to false will cause error messages to be displayed on standard output.</remarks>
		public bool ExceptionsOnError
		{
			get
			{
				return _exceptionsOnError;
			}
			set
			{
				_exceptionsOnError = value;
			}
		}

		/// <summary>
		/// Indicates whether exceptions should be thrown when a warning is generated.
		/// </summary>
		private bool _exceptionsOnWarnings;

		/// <summary>
		/// Gets or sets whether Exceptions should be thrown when an warning is generated.
		/// </summary>
		/// <remarks>The default value is true. Setting this to false will cause warnings to be displayed on standard output.</remarks>
		public bool ExceptionsOnWarnings
		{
			get
			{
				return _exceptionsOnWarnings;
			}
			set
			{
				_exceptionsOnWarnings = value;
			}
		}

		/// <summary>
		/// The total number of warnings generated while parsing the RDF/XML.
		/// </summary>
		private long _warningCount;
		
		/// <summary>
		/// Gets the total number of warnings generated while parsing the RDF/XML
		/// </summary>
		public long Warnings
		{
			get
			{
				return _warningCount;
			}
		}

		/// <summary>
		/// Called by the parser when an error is encountered.
		/// </summary>
		/// <param name="msg">The error message associated with the error</param>
		/// <exception cref="InvalidRdfException">ExceptionsOnError is set to true</exception>
		private void OnError(string msg)
		{
			_errorCount++;
			if(_exceptionsOnError)
				throw (new InvalidRdfException(msg));
			Console.WriteLine("Error: "+msg);
		}

		/// <summary>
		/// Called by the parser when an error is encountered.
		/// </summary>
		/// <param name="e">The exception to throw.</param>
		/// <remarks>If ExceptionsOnError is set to true then the specified Exception is thrown. 
		/// If ExceptionsOnError is set to false then the error message from the exception is output to standard output</remarks>
		private void OnError(Exception e)
		{
			_errorCount++;
			if(_exceptionsOnError)
				throw (e);
			Console.WriteLine("Error: "+e.Message);
		}

		/// <summary>
		/// Called by the parser when a warning is generated.
		/// </summary>
		/// <param name="msg">The message associated with the warning</param>
		/// <exception cref="InvalidRdfException">ExceptionsOnWarnings is set to true</exception>
		private void OnWarning(string msg)
		{
			_warningCount++;
			if(_exceptionsOnWarnings)
				throw (new InvalidRdfException(msg));
			Console.WriteLine("Warning: "+msg);
		}
		/// <summary>
		/// Called by the parser when a warning is generated. 
		/// </summary>
		/// <param name="e">The exception to throw if ExceptionsOnWarnings is true</param>
		/// <remarks>If ExceptionsOnWarnings is set to true then the specified exception is thrown.
		/// If ExceptionsOnWarnings is set to false then the error message from the exception is output to standard output</remarks>
		private void OnWarning(Exception e)
		{
			_warningCount++;
			if(_exceptionsOnWarnings)
				throw (e);
			Console.WriteLine("Warning: "+e.Message);
		}
		
		/// <summary>
		/// The XML document
		/// </summary>
		private XmlDocument _xmlDoc;
		
		/// <summary>
		/// Initializes an instance of the RdfParser class
		/// </summary>
		public RdfParser()
		{
			_rdfGraph = null;
			_errorCount = 0;
			_warningCount = 0;
			_exceptionsOnError = true;
			_exceptionsOnWarnings = true;
			_newID = 10000;
			string[] rdfXmlAttrs = {"rdf:about", "rdf:resource", "rdf:parseType", "rdf:ID", "rdf:nodeID", "rdf:bagID", "rdf:datatype", "xml:lang"};	
			_rdfXmlProperties = new Hashtable();
			for(int i=0;i<rdfXmlAttrs.Length;i++)
				_rdfXmlProperties.Add(rdfXmlAttrs[i],rdfXmlAttrs[i]);

			string[] rdfProps = {"rdf:resource", "rdf:parseType", "rdf:ID", "rdf:nodeID", "rdf:bagID", "rdf:datatype", "rdf:about"};
			_rdfProperties = new Hashtable();
			for(int i=0;i<rdfProps.Length;i++)
				_rdfProperties.Add(rdfProps[i], rdfProps[i]);

		}

		/// <summary>
		/// The RDF Graph object
		/// </summary>
		private RdfGraph _rdfGraph;
		
		/// <summary>
		/// Builds an RdfGraph from an RDF/XML serialization
		/// </summary>
		/// <param name="uri">The URI of the RDF document to parse</param>
		/// <returns>An object that implements the IRdfGraph interface</returns>
		public IRdfGraph ParseRdf(String uri)
		{
			_xmlDoc = new XmlDocument();
			try
			{
				_xmlDoc.Load(uri);
			}
			catch(XmlException xe)
			{
				OnError(xe);
				return null;
			}
			catch(Exception e)
			{
				OnError(e);
				return null;
			}
			//Start with the root
			XmlElement root = _xmlDoc.DocumentElement;
			if(root.Name != "rdf:RDF")
			{
				OnError("Not an RDF Document.");
				return null;
			}
			//Now create the RDFGraph
			_rdfGraph = new RdfGraph();

			//its an RDF Document so now get the namespace info
			for(int i=0;i<root.Attributes.Count;i++)
			{
				try
				{
					_rdfGraph.NameSpaces[root.Attributes[i].Name] = root.Attributes[i].Value;
				}
				catch(ArgumentNullException ane)
				{
					OnWarning(ane);
				}
				catch(ArgumentException ine)
				{
					OnWarning(ine);
				}
				
			}
			try
			{
				_rdfGraph.NameSpaces["xml:base"] = _xmlDoc.BaseURI;
			}
			catch(ArgumentException)
			{
				//The xml:base namespace has already been added. Do Nothing.
			}
			
			if(root.HasChildNodes)
			{
				for(int i=0;i<root.ChildNodes.Count;i++)
					ProcessNode(root.ChildNodes[i], true,null);
			}
		
			return _rdfGraph;
		}

		/// <summary>
		/// Adds an RdfNode object with the specified Uri to the Graph.
		/// </summary>
		/// <param name="nodeUri">The Uri of the node to add</param>
		/// <returns>The newly added RdfNode object</returns>
		/// <remarks>If the node with the specified Uri does not already exist it is created and added.
		/// If the node exists in the graph then a reference to the existing node is returned.
		/// If the specified Uri is null or blank a new blank Uri is created. 
		/// If the specified Uri is a relatve Uri then it is converted to an absolute Uri by prefacing it with the value given by the xml:base attribute in the rdf:RDF element</remarks>
		private RdfNode AddNodeToGraph(string nodeUri)
		{
			if((nodeUri == null) || (nodeUri.Length == 0))
				return (RdfNode)_rdfGraph.AddNode(GetBlankNodeUri(null));
			return (RdfNode)_rdfGraph.AddNode(MakeUri(nodeUri));
		}

		/// <summary>
		/// Overwrites the oldNode with the new node.
		/// </summary>
		/// <param name="oldNode">The node from which the edges are to be removed.</param>
		/// <param name="newNode">The node to connect all the edges to</param>
		/// <exception cref="ArgumentNullException">oldNode  is null</exception>
		/// <exception cref="ArgumentNullException">newNode  is null</exception>
		/// <remarks>This method moves all the edges from the old node to the new node. 
		/// The old node is not removed from the graph. </remarks>
		private void OverwriteNode(RdfNode oldNode, RdfNode newNode)
		{
			if((oldNode == null) || (newNode == null))
				throw (new ArgumentNullException());
			for(int i=0;i<oldNode.ParentEdges.Count;i++)
			{
				oldNode.ParentEdges[i].AttachChildNode(newNode);
			}

			for(int i=0;i<oldNode.ChildEdges.Count;i++)
			{
				oldNode.ChildEdges[i].AttachParentNode(newNode);
			}
		}

		/// <summary>
		/// Adds an RDF Container of type rdf:Alt to the graph</summary>
		/// <param name="nodeUri">The Uri of the container.</param>
		/// <returns>Returns a reference to the newly added container.</returns>
		/// <exception cref="UriFormatException">The specified nodeUri  is not a well formed Uri.</exception>
		/// <remarks>If a node with the specified Uri exists then it is converted to an RdfAlt (if required) and returned. 
		/// If the container does not exist a new one is created</remarks>
		private RdfAlt AddAltToGraph(string nodeUri)
		{
			//if the usr is null then create a blank node uri
			if(nodeUri == null)
				nodeUri = GetBlankNodeUri(null);
			RdfNode node = (RdfNode)_rdfGraph[nodeUri];
			if((node != null) && (node is RdfAlt))
				return (RdfAlt)node;
			RdfNode typeNode = AddNodeToGraph(RdfNamespaceCollection.RdfNamespace+"Alt");

			if(node == null)
			{
				node = new RdfAlt(nodeUri,typeNode);
				_rdfGraph.AddNode(node);
				return (RdfAlt)node;
			}

			RdfAlt newNode = new RdfAlt(nodeUri,typeNode);
			OverwriteNode(node, newNode);
			_rdfGraph.Nodes.Remove(node);
			_rdfGraph.AddNode(newNode);
			return newNode;
		}

		/// <summary>
		/// Adds an RdfStatement to the RDF graph
		/// </summary>
		/// <param name="statementUri">The Uri of the statement.</param>
		/// <exception cref="UriFormatException">The specified statementUri  is not a well formed Uri.</exception>
		/// <returns>A reference to the newly added statement</returns>
		/// <remarks>If a node with the specified Uri exists then it is converted to an RdfStatement (if required) and returned. 
		/// If the statement does not exist a new one is created</remarks>
		private RdfStatement AddStatementToGraph(string statementUri)
		{
			RdfNode node = (RdfNode)_rdfGraph[statementUri];
			if((node != null) && (node is RdfStatement))
				return (RdfStatement)node;
			RdfNode typeNode = AddNodeToGraph(RdfNamespaceCollection.RdfNamespace+"Statement");

			if(node == null)
			{
				node = new RdfStatement(statementUri,typeNode);
				_rdfGraph.AddNode(node);
				return (RdfStatement)node;
			}
			RdfStatement newNode = new RdfStatement(statementUri,typeNode);
			OverwriteNode(node, newNode);
			_rdfGraph.Nodes.Remove(node);
			_rdfGraph.AddNode(newNode);
			return newNode;
		}

		/// <summary>
		/// Adds an RDF Container of type rdf:Bag to the graph</summary>
		/// <param name="nodeUri">The Uri of the container.</param>
		/// <returns>Returns a reference to the newly added container.</returns>
		/// <exception cref="UriFormatException">The specified nodeUri  is not a well formed Uri.</exception>
		/// <remarks>If a node with the specified Uri exists then it is converted to an RdfBag (if required) and returned. 
		/// If the container does not exist a new one is created</remarks>
		private RdfBag AddBagToGraph(string nodeUri)
		{
			//if the usr is null then create a blank node uri
			if(nodeUri == null)
				nodeUri = GetBlankNodeUri(null);
			RdfNode node = (RdfNode)_rdfGraph[nodeUri];
			if((node != null) && (node is RdfBag))
				return (RdfBag)node;
			RdfNode typeNode = AddNodeToGraph(RdfNamespaceCollection.RdfNamespace+"Bag");
			if(node == null)
			{
				node = new RdfBag(nodeUri,typeNode);
				_rdfGraph.AddNode(node);
				return (RdfBag)node;
			}
			RdfBag newNode = new RdfBag(nodeUri, typeNode);
			OverwriteNode(node, newNode);
			_rdfGraph.Nodes.Remove(node);
			_rdfGraph.AddNode(newNode);
			return newNode;
		}

		/// <summary>
		/// Adds an RDF Container of type rdf:Seq to the graph</summary>
		/// <param name="nodeUri">The Uri of the container.</param>
		/// <returns>Returns a reference to the newly added container.</returns>
		/// <exception cref="UriFormatException">The specified nodeUri  is not a well formed Uri.</exception>
		/// <remarks>If a node with the specified Uri exists then it is converted to an RdfSeq (if required) and returned. 
		/// If the container does not exist a new one is created</remarks>
		private RdfSeq AddSeqToGraph(string nodeUri)
		{
			//if the usr is null then create a blank node uri
			if(nodeUri == null)
				nodeUri = GetBlankNodeUri(null);
			RdfNode node = (RdfNode)_rdfGraph[nodeUri];
			if((node != null) && (node is RdfSeq))
				return (RdfSeq)node;
			RdfNode typeNode = AddNodeToGraph(RdfNamespaceCollection.RdfNamespace+"Seq");

			if(node == null)
			{
				node = new RdfSeq(nodeUri,typeNode);
				_rdfGraph.AddNode(node);
				return (RdfSeq)node;
			}

			RdfSeq newNode = new RdfSeq(nodeUri, typeNode);
			OverwriteNode(node, newNode);
			_rdfGraph.Nodes.Remove(node);
			_rdfGraph.AddNode(newNode);
			return newNode;
		}

		/// <summary>
		/// Adds an  RdfLiteral object to the RDF Graph     
		/// </summary>
		/// <param name="literalValue">The literal value of the node to add</param>
		/// <param name="langID">A string containing the language identifier for this Literal</param>
		/// <param name="datatypeUri">A string containing the URI that specifies the datatype for thsi literal.</param>
		/// <exception cref="UriFormatException">The specified datatypeUri  is not a null reference and is not a well formed URI</exception>
		/// <returns>The newly added RdfLiteral object</returns>
		/// <remarks>If a literal with the specified value, langID and datatype exists then a reference to the existing literal is returned. 
		/// If the Literal is not found a new one is created.</remarks>
		private RdfLiteral AddLiteralToGraph(string literalValue, string langID, string datatypeUri)
		{
			return (RdfLiteral)_rdfGraph.AddLiteral(literalValue,langID,datatypeUri);
		}

		/// <summary>
		/// Creates a node with the specified nodeUri
		/// </summary>
		/// <param name="node">The XmlNode element specifying the type of the RdfNode</param>
		/// <param name="nodeUri">The Uri of the new node. It may be null, blank or a relative Uri.</param>
		/// <returns>A reference to the newly created TypedNode</returns>
		/// <exception cref="ArgumentNullException">node  is null a null reference</exception>
		/// <remarks>The new node created has a child edge with ID rdf:type from this node pointing 
		/// to a node with URI specified by the name of the given XmlNode element.
		/// If the specified URI is null or empty a new blank node Uri is created. 
		/// If the Uri is a relative URI it is converted to an absolute URI by prefixing it with the value given by xml:base.</remarks>
		private RdfNode CreateTypedNode(XmlNode node, string nodeUri)
		{
			if(node == null)
				throw (new ArgumentNullException());
			String namespaceUri = node.NamespaceURI;
			String localName = node.LocalName;

			//add the typed node to the graph
			RdfNode rNode = AddNodeToGraph(nodeUri);
			
			//add the child node of the typedNode i.e. the node that indicates the type
			RdfNode childNode = AddNodeToGraph(namespaceUri+localName);
			
			//create an rdf:type edge
			RdfEdge rEdge = new RdfEdge(RdfNamespaceCollection.RdfNamespace+"type");
			
			//connect all three
			rNode.AttachChildEdge(rEdge);
			rEdge.AttachChildNode(childNode);
			return rNode;
		}
	
		/// <summary>
		/// Processes each node in the XML document. 
		/// </summary>
		/// <param name="node">The XmlNode to process.</param>
		/// <param name="bIsRdfNode">Indicates if we are about to process a node or an edge in the RDF Graph</param>
		/// <param name="parent">The parent of the current node.</param>
		/// <returns>Returns a reference to the new node or edge created</returns>
		/// <exception cref="InvalidRdfException">The RDF Syntax encountered is invalid</exception>
		/// <exception cref="RdfParseException">The parser encountered a fatal parse error</exception>
		/// <remarks>This method is called recursively to build the RDF graph.
		/// The parent is an object of type RdfNode or RdfEdge.
		/// An RdfParseException is thrown to indicate a parser error.</remarks>
		private Object ProcessNode(XmlNode node, bool bIsRdfNode, Object parent)
		{
			//if the node is a comment or anything else then totally ignore
			if((node.NodeType == XmlNodeType.Comment) || (node.NodeType == XmlNodeType.None) || (node.NodeType == XmlNodeType.Whitespace) || (node.NodeType == XmlNodeType.SignificantWhitespace))
				return true;
			RdfNode rNode = null;
			RdfEdge rEdge = null;
			if(bIsRdfNode) //we are processing an Rdf node in the XML Dom tree. i.e. a node wrt to the striped RDF syntax
			{
				RdfEdge parentRdfEdge = null;
				string langID = null;
				if(parent != null)
				{
					parentRdfEdge = (RdfEdge)parent;
					langID = parentRdfEdge.LangID;
				}
				if(node.NodeType == XmlNodeType.Element)
				{
					//first parse the Rdf attributes rdf:about, rdf:ID, rdf:nodeID
					string nodeID = ParseRdfAttributes(node);

					rNode = ParseRdfSyntax(node,nodeID);
					if(rNode == null) 		//its not standard RDF Syntax so create an edge (rdf:type) from this node
						rNode = CreateTypedNode(node,nodeID);

					//try and get the xml:lang attribute or inherit the xml:lang attribute from the parent if applicable
					XmlNode langAttr = node.Attributes["xml:lang"];
					if(langAttr != null)
						rNode.LangID = langAttr.Value;
					else
						rNode.LangID = langID;

					//process the regular (Non-Rdf) attributes of this node
					ParseRegularAttributes(node, rNode);
				}
				else if(node.NodeType == XmlNodeType.Text)
				{
					//its a literal
					//so get the lang ID and the datatype from the parent node
					string datatypeUri = GetDatatypeUri(node.ParentNode);
					rNode = AddLiteralToGraph(node.Value,langID, datatypeUri);
				}

				if(parentRdfEdge != null)
					parentRdfEdge.AttachChildNode(rNode);

				if(node.HasChildNodes)
				{
					for(int i=0;i<node.ChildNodes.Count;i++)
						ProcessNode(node.ChildNodes[i], !bIsRdfNode, rNode);
				}
				//Now parse the attributes that can only be processed on the way back the DOM tree such as rdf:bagID
				if(node.NodeType == XmlNodeType.Element)
					ParseReificationAttributes(node, rNode);
			}
			else //we are processing an edge in the RDF Graph. i.e. an edge wrt to the Striped RDF Syntax
			{
				rEdge = new RdfEdge();
				if(parent == null)
					throw(new RdfParseException("Parent node of an Edge cannot be null"));

				RdfNode parentRdfNode = (RdfNode)parent;
				//process the xml:lang attribute if applicable
				rEdge.LangID = parentRdfNode.LangID;
				
				if(!ParseRdfSyntax(node,rEdge, parentRdfNode))
				{
					rEdge.ID = node.NamespaceURI+node.LocalName;
					rEdge.AttachParentNode(parentRdfNode);
				}
				
				if(ParseRdfAttributes(node,rEdge)) //if the process attributes method returns true then process the children of this node
				{
					if(node.HasChildNodes)
					{
						for(int i=0;i<node.ChildNodes.Count;i++)
							ProcessNode(node.ChildNodes[i], !bIsRdfNode, rEdge);
					}
				}

				//if the edge is dangling then put a blank node on it.
				if(rEdge.ChildNode == null)
					rEdge.AttachChildNode(AddNodeToGraph(GetBlankNodeUri(null)));
				ParseRegularAttributes(node,rEdge);
				//Now process the rdf:ID and rdf:bagID attributes to complete reification 
				ParseReificationAttributes(node, rEdge);
			}
			if(bIsRdfNode)
				return rNode;
			else
				return rEdge;
		}

		/// <summary>
		/// Gets the rdf:datatype Uri from the specified XML element
		/// </summary>
		/// <param name="node">The XmlNode from which to extract the rdf:datatype attribute</param>
		/// <returns>A string representing the datatype Uri. A null reference is returned if the rdf:datatype attribut is not found.</returns>
		/// <remarks>This method looks for the rdf:datatype attribute on the specified XML element and returns the value.</remarks>
		private string GetDatatypeUri(XmlNode node)
		{
			XmlNode datatypeAttr = node.Attributes["rdf:datatype"]; //kingtiny must validate this
			if(datatypeAttr != null)
				return MakeUri(datatypeAttr.Value);
			return null;
		}

		/// <summary>
		/// Determines whether the XML Node is a property that is part of the RDF or XML syntax
		/// </summary>
		/// <param name="prop">An XmlNode</param>
		/// <returns>True if the property is part of the RDF or XML syntax</returns>
		private bool IsRdfXmlProperty(XmlNode prop)
		{
			if(_rdfXmlProperties[prop.Name] != null)
				return true;
			if((prop.Prefix == "rdf") || (prop.Prefix == "xml"))
				OnWarning("Unknown RDF or XML property "+prop.LocalName);
			return false;
		}

		/// <summary>
		/// Determines whether the XML Node is a property that is part of the RDF Syntax
		/// </summary>
		/// <param name="prop">An XmlNode</param>
		/// <returns>True if the property is part of the RDF syntax</returns>
		private bool IsRdfProperty(XmlNode prop)
		{
			if(_rdfProperties[prop.Name] != null)
				return true;
			if(prop.Prefix == "rdf")
				OnWarning("Unknown RDF property "+prop.Value);
			return false;
		}

		/// <summary>
		/// Creates a Bag of reified statements
		/// </summary>
		/// <param name="bagID">The ID of the bag. It may be empty, null or a relative Uri.</param>
		/// <param name="rNode">The subject of all the reified statements</param>
		/// <remarks>If the specified ID is null or empty a new blank node URI is created.
		/// If it is a relative URI then it is converted to an absolute by prefixing it with the value of xml:base</remarks>
		private void CreateReifiedBag(string bagID, RdfNode rNode)
		{
			RdfBag bag = null;
			if((bagID == null) || (bagID.Length == 0))
				bag = AddBagToGraph(GetBlankNodeUri(null));
			else
				bag = AddBagToGraph(MakeUri(bagID));
			if(rNode == null)
				return;
			//create a statement for each edge
			for(int i=0;i<rNode.ChildEdges.Count;i++)
			{
				RdfEdge childEdge = (RdfEdge)rNode.ChildEdges[i];
				RdfStatement statement = CreateReifiedStatement(GetBlankNodeUri(null),rNode,AddNodeToGraph(childEdge.ID),(RdfNode)childEdge.ChildNode);
				bag.Add(statement);
			}
		}

		/// <summary>
		/// Creates a bag of Reified Statements
		/// </summary>
		/// <param name="bagID">The ID of the bag</param>
		/// <param name="rEdge">The parent edge of the subject of each reified statement</param>
		/// <remarks>If the specified ID is null or empty a new blank node URI is created.
		/// If it is a relative URI then it is converted to an absolute by prefixing it with the value of xml:base</remarks>
		private void CreateReifiedBag(string bagID, RdfEdge rEdge)
		{
			RdfNode rNode = (RdfNode)rEdge.ChildNode;
			CreateReifiedBag(bagID, rNode);
		}

		/// <summary>
		/// Parses the reification attribute rdf:bagID as it appears on a node in the RDF Graph and creates a bag of reified statements
		/// </summary>
		/// <param name="node">The node on which the attribute appears</param>
		/// <param name="rNode">The subject of all the reified statements in the bag</param>
		private void ParseReificationAttributes(XmlNode node, RdfNode rNode)
		{
			XmlNode attr = node.Attributes["rdf:bagID"];
			if(attr != null)
			{
				CreateReifiedBag(attr.Value,rNode);
			}
		}

		/// <summary>
		/// Parses the reification attributes rdf:ID and rdf:bagID as they appear on an edge in the RDF graph.
		/// </summary>
		/// <param name="node">The XmlNode containing the attributes</param>
		/// <param name="rEdge">The predicate if the rdf:ID attribute is found and the parent of the subject node if the rdf:bagID attribut is found.</param>
		/// <remarks>Reifies the triple with predicate rEdge if the attribute rdf:ID is found and 
		/// creates a bag of reified statements if the rdf:bagID attribute is found</remarks>
		private void ParseReificationAttributes(XmlNode node, RdfEdge rEdge)
		{
			XmlNode attr = node.Attributes["rdf:ID"];
			if(attr != null)
			{
				RdfNode predicate = AddNodeToGraph(rEdge.ID);
				CreateReifiedStatement(attr.Value,(RdfNode)rEdge.ParentNode,predicate,(RdfNode)rEdge.ChildNode);
			}
			attr = node.Attributes["rdf:bagID"];
			if(attr != null)
			{
				CreateReifiedBag(attr.Value,rEdge);
			}
		}

		/// <summary>
		/// Parses the attributes appearing on a node that are not part of the RDF or XML namespace
		/// </summary>
		/// <param name="node">The XmlNode on which the attributes appear</param>
		/// <param name="rNode">The RdfNode to which the attributes must be applied</param>
		private void ParseRegularAttributes(XmlNode node, RdfNode rNode)
		{
			for(int i=0;i<node.Attributes.Count;i++)
			{
				if(!IsRdfXmlProperty(node.Attributes[i]))
				{
					RdfLiteral literal = AddLiteralToGraph(node.Attributes[i].Value,rNode.LangID,GetDatatypeUri(node));
					RdfEdge rEdge = new RdfEdge(node.Attributes[i].NamespaceURI+node.Attributes[i].LocalName);
					rNode.AttachChildEdge(rEdge);
					rEdge.AttachChildNode(literal);
				}
			}
		}

		/// <summary>
		/// Process the attributes appearing on an edge that are not part of the RDF or XML namespace
		/// </summary>
		/// <param name="node"></param>
		/// <param name="rEdge"></param>
		private void ParseRegularAttributes(XmlNode node, RdfEdge rEdge)
		{
			//go through all the attributes
			for(int i=0;i<node.Attributes.Count;i++)
			{
				if(!IsRdfProperty(node.Attributes[i]))
				{
					//make an edge from the child of rEdge
					RdfEdge childEdge = new RdfEdge();
					childEdge.ID = node.Attributes[i].NamespaceURI+node.Attributes[i].LocalName;
					rEdge.ChildNode.AttachChildEdge(childEdge);
					//create a new RdfLiteral
					RdfLiteral childNode = AddLiteralToGraph(node.Attributes[i].Value,rEdge.LangID,GetDatatypeUri(node));
					//attach it to the edge
					childEdge.AttachChildNode(childNode);
				}
			}
		}

		/// <summary>
		/// Parses the RDF attributes rdf:about, rdf:ID and rdf:nodeID
		/// </summary>
		/// <param name="node">The XmlNode on which the attributes appear</param>
		/// <returns>A Uri string with the ID specified by the attributes. Null if none of the three attributes are found</returns>
		private string ParseRdfAttributes(XmlNode node)
		{
			int attrFound = 0;
			XmlNode attr = node.Attributes["rdf:about"];
			string retVal = null;
			if(attr != null)
			{
				retVal = MakeUri(attr.Value);
				attrFound = 1;
			}
			attr = node.Attributes["rdf:ID"];
			if(attr != null)
			{
				retVal = MakeUri(attr.Value);
				attrFound += 2;
			}
			attr = node.Attributes["rdf:nodeID"];
			if(attr != null)
			{
				retVal = GetBlankNodeUri(attr.Value);
				attrFound += 4;
			}
			attr = node.Attributes["xml:lang"];
			switch(attrFound)
			{
				case 3:
					OnError("Cannot use rdf:about and rdf:ID together");
					break;
				case 5:
					OnError("Cannot use rdf:about and rdf:nodeID together");
					break;
				case 6:
					OnError("Cannot use rdf:ID and rdf:nodeID together");
					break;
				case 7:
					OnError("Cannot use rdf:about, rdf:ID and rdf:nodeID together");
					break;
			}
			return retVal;
		}

		/// <summary>
		/// Parses the RDF Attributes rdf:resource, rdf:nodeID, rdf:parseType, and xml:lang as they appear on an edge
		/// </summary>
		/// <param name="node">The XmlNode on which the attributes appear</param>
		/// <param name="rEdge">The RdfEdge to which the attributes must be applied</param>
		/// <returns>True if the children of the specified XML Node should be parsed</returns>
		private bool ParseRdfAttributes(XmlNode node, RdfEdge rEdge)
		{
			int attrFound = 0;
			bool parseChildren = true;
			XmlNode attr = node.Attributes["rdf:resource"];
			if(attr != null)
			{
				ProcessRdfResource(rEdge, attr.Value);
				attrFound = 1;
				parseChildren = false;
			}
			attr = node.Attributes["rdf:nodeID"];
			if(attr != null)
			{
				ProcessRdfNodeID(rEdge, attr.Value);
				attrFound += 2;
				parseChildren = false;
			}
			attr = node.Attributes["rdf:parseType"];
			if(attr != null)
			{
				ProcessRdfParseType(node,rEdge,attr.Value);
				attrFound += 4;
				parseChildren = false;
			}
			attr = node.Attributes["xml:lang"];
			if(attr != null)
			{
				rEdge.LangID = attr.Value;
			}
			switch(attrFound)
			{
				case 3:
					OnError("Cannot use rdf:resource and rdf:nodeID together");
					break;
				case 5:
					OnError("Cannot use rdf:resource and rdf:parseType together");
					break;
				case 6:
					OnError("Cannot use rdf:nodeID and rdf:parseType together");
					break;
				case 7:
					OnError("Cannot use rdf:resource, rdf:nodeID and rdf:parseType together");
					break;
			}
			return parseChildren;
		}

		/// <summary>
		/// Gets a URI string for a new blank node
		/// </summary>
		/// <param name="baseID">The base ID from which the Uri must be created.</param>
		/// <returns>A string containing a well formed URI</returns>
		private string GetBlankNodeUri(string baseID)
		{
			if((baseID == null) || (baseID.Length == 0))
				return "blankID:"+_newID++;
			else
				return "blankID:"+baseID;
		}

		/// <summary>
		/// Process the rdf:nodeID attribute found on the RDF edge
		/// </summary>
		/// <param name="rEdge">The edge to which the rdf:nodeID attribute must be applied</param>
		/// <param name="baseNodeID">The ID specified by the rdf:nodeID attribute</param>
		private void ProcessRdfNodeID(RdfEdge rEdge, string baseNodeID)
		{
			string nodeID = GetBlankNodeUri(baseNodeID);
			RdfNode rNode = AddNodeToGraph(nodeID);
			rEdge.AttachChildNode(rNode);
		}

		/// <summary>
		/// Process the rdf:resource attribute
		/// </summary>
		/// <param name="rEdge">The edge to which the attribute must be applied</param>
		/// <param name="resourceUri">The URI specified by the rdf:resource attribute</param>
		/// <remarks>If the specified Uri is null or empty a new blank node URI is created. 
		/// If it is a relative URI then it is converted to an absolute URI by prefixing it with the value given by xml:base</remarks>
		private void ProcessRdfResource(RdfEdge rEdge, string resourceUri)
		{
			string nodeID = MakeUri(resourceUri);
			RdfNode rNode = AddNodeToGraph(nodeID);
			rEdge.AttachChildNode(rNode);
		}

		/// <summary>
		/// Makes a URI string from the specified ID.
		/// </summary>
		/// <param name="id">The ID to convert to a URI string</param>
		/// <returns>A string containing a well formed URI</returns>
		private string MakeUri(string id)
		{
			try
			{
				Uri tempUri = new Uri(id);
				return id;
			}
			catch(UriFormatException)
			{
				if(id != null)
					return _rdfGraph.NameSpaces["xml:base"]+id;	
				else
					return _rdfGraph.NameSpaces["xml:base"];	
			}
		}

		/// <summary>
		/// Processes the rdf:parseType attribute
		/// </summary>
		/// <param name="node">The XML Node on which the attribute appears</param>
		/// <param name="rEdge">The edge to which the attribute must be applied</param>
		/// <param name="parseType">The parse type as specified by the value of the attribute</param>
		private void ProcessRdfParseType(XmlNode node, RdfEdge rEdge, string parseType)
		{
			if(parseType == "Resource")
			{
				//its an rdf:parseType="Resource" so we must now parse its children
				//the children of this xmlNode are now EDGES in the RDF graph that 
				//extend out from the dest node of this edge i.e. the rdfNode created above
							
				//create a new node that will be the child of this edge
				//create a new node ID
				string nodeID = GetBlankNodeUri(null);
				RdfNode rNode = AddNodeToGraph(nodeID);
				//attach this node to the edge
				rEdge.AttachChildNode(rNode);
				if(node.HasChildNodes)
				{
					for(int i=0;i<node.ChildNodes.Count;i++)
						ProcessNode(node.ChildNodes[i], false, rNode);
				}
			}
			else if(parseType == "Literal")
			{
				//its an rdf:parseType="Literal" so all the xml below this node 
				//will be the content of the dest node i.e. rdfNode
				//create a new node that will be the child of this edge
				//create a new node ID
				string literalValue = node.InnerXml;
				RdfLiteral literal = AddLiteralToGraph(literalValue,rEdge.LangID,GetDatatypeUri(node));
				//attach this node to the edge
				rEdge.AttachChildNode(literal);
			}
			else if(parseType == "Collection")
			{
				//its a Collection so make a cons list
				//get the children of this node
				RdfNode rNode = BuildCollection(node);
				//connect the collection to the edge
				rEdge.AttachChildNode(rNode);
			}
			else
			{
				OnError("Unknown parseType "+parseType);
			}
		}

		/// <summary>
		/// Adds an edge and a child node to the specified parent node 
		/// </summary>
		/// <param name="parentNode">The node to which the edge should be added</param>
		/// <param name="rdfType">The ID of the node. This param is prefaced with the rdf Uri to convert it to an absolute Uri</param>
		private void AddTypeStatement(RdfNode parentNode, string rdfType)
		{
			if(parentNode == null)
				return;
			
			//create the typeEdge
			RdfEdge typeEdge = new RdfEdge();
			typeEdge.ID = RdfNamespaceCollection.RdfNamespace+"type";

			//find/create the typeNode
			RdfNode typeNode = AddNodeToGraph(RdfNamespaceCollection.RdfNamespace+rdfType);
			
			//connect all three
			parentNode.AttachChildEdge(typeEdge);
			typeEdge.AttachChildNode(typeNode);
		}

		/// <summary>
		/// Builds an RDF Collection.
		/// </summary>
		/// <param name="propertyNode">The XML Node containing the children that will form the members of this list</param>
		/// <returns>The head of the collection. This is the first member of the list or a nil node if the list is empty</returns>
		private RdfNode BuildCollection(XmlNode propertyNode)
		{
			RdfNode rNode;
			RdfNode collHead = null;
			RdfEdge restEdge = null;
			RdfEdge firstEdge;
			for(int i=0;i<propertyNode.ChildNodes.Count;i++)
			{
				//build the blank node that is a member of this list
				string nodeID = GetBlankNodeUri(null);
				rNode = AddNodeToGraph(nodeID);
				//if this is the first element of the cons list then set this as the head of the collection so we may return it later
				if(i == 0)
					collHead = rNode;

				//add an outgoing edge with name rdf:type from rdfNode to rdf:List
				AddTypeStatement(rNode, "List");
										
				//create a new first edge
				firstEdge = new RdfEdge(RdfNamespaceCollection.RdfNamespace+"first");
				rNode.AttachChildEdge(firstEdge);
				RdfNode firstNode =	(RdfNode)ProcessNode(propertyNode.ChildNodes[i],true,firstEdge);
				firstEdge.AttachChildNode(firstNode);
		
				if(restEdge != null)
					rNode.AttachParentEdge(restEdge);

				//make the new rest Edge
				restEdge = new RdfEdge();
				restEdge.ID = RdfNamespaceCollection.RdfNamespace+"rest";
				restEdge.AttachParentNode(rNode);
			}
			//ok we have exited the loop or bypassed it because there are no children
			
			rNode = AddNodeToGraph(RdfNamespaceCollection.RdfNamespace+"nil");
			if(restEdge != null)
				rNode.AttachParentEdge(restEdge);
					
			if(collHead == null)
				collHead = rNode;
			return collHead;
		}
		
		/// <summary>
		/// Determines whether the specified name is an XML name
		/// </summary>
		/// <param name="name">A Name</param>
		/// <returns>True if the name is an XML name</returns>
		private bool IsXmlName(string name)
		{
			if(name.IndexOfAny(new char[]{'\\',':','/'}) == -1)
				return true;
			return false;
		}

		/// <summary>
		/// Creates a Reified RDF statement
		/// </summary>
		/// <param name="statementID">The ID of the statement</param>
		/// <param name="subject">The subject of the statement</param>
		/// <param name="predicate">The predicate of Statement</param>
		/// <param name="obj">The Object of the statement</param>
		/// <returns>A reference to the newly created statement</returns>
		private RdfStatement CreateReifiedStatement(string statementID, RdfNode subject, RdfNode predicate, RdfNode obj)
		{
			RdfStatement statement = AddStatementToGraph(MakeUri(statementID));

			//add the subject to the statement
			statement.RdfSubject = subject;

			//add the predicate to the statement
			statement.RdfPredicate = predicate;
			
			//add the object to the statement
			statement.RdfObject = obj;

			return statement;
		}

		/// <summary>
		/// Parses the RDF syntax [rdf:li, rdf:_n, rdf:type] on an edge.
		/// </summary>
		/// <param name="node">The XML Node representing the RDF edge</param>
		/// <param name="rEdge">The edge</param>
		/// <param name="parentNode">The parent RDF node</param>
		/// <returns>True if the edge was given an ID and attached to the parent node</returns>
		private bool ParseRdfSyntax(XmlNode node, RdfEdge rEdge, RdfNode parentNode)
		{
			//first get the NameSpace URI, NameSpace prefix and the LocalName for the node
			String nameSpaceURI = node.NamespaceURI;
			String nameSpacePrefix = node.Prefix;
			String localName = node.LocalName;
			
			if(nameSpaceURI == RdfNamespaceCollection.RdfNamespace)
			{
				if(localName=="li")						//process the rdf:li
				{
					//check if the parent node is a collection
					if(parentNode is RdfContainer)
					{
						rEdge.ID = RdfNamespaceCollection.RdfNamespace+"_"+(((RdfContainer)parentNode).Count+1);
					}
					else
					{
						rEdge.ID = RdfNamespaceCollection.RdfNamespace+"_"+(parentNode.ChildEdges.Count+1);
						OnWarning("Parent node of list edge [rdf:li] is not an Rdf Container.");
					}
					rEdge.AttachParentNode(parentNode);
					return true;
				}
				if(localName == "type")					//process the rdf:type
				{
					rEdge.ID = RdfNamespaceCollection.RdfNamespace+"type";
					rEdge.AttachParentNode(parentNode);
					return true;
				}
				if(IsListMember(localName))				//process the rdf:_n
				{
					rEdge.ID = RdfNamespaceCollection.RdfNamespace+localName;
					rEdge.AttachParentNode(parentNode);
					return true;
				}
				OnWarning("Unknown Rdf element rdf:"+localName);
			}
			return false;
		}

		/// <summary>
		/// Determines whether the edgename is a list member with format rdf:_n
		/// </summary>
		/// <param name="edgeName">The name of the edge</param>
		/// <returns>True if the name represents a list member</returns>
		private bool IsListMember(string edgeName)
		{
			if(edgeName.StartsWith("_"))
			{
				try
				{
					Int32.Parse(edgeName.Substring(1));
				}
				catch(FormatException)
				{
					return false;
				}
				return true;
			}
			return false;
		}

		/// <summary>
		/// Process the RDF Syntax [rdf:Description, rdf:Bag, rdf:Seq, rdf:Alt] on an RDF Node
		/// </summary>
		/// <param name="node">The XmlNode representing the RDF Node</param>
		/// <param name="nodeID">The ID to be assigned to the RDF node</param>
		/// <returns>The newly created RDF node or null if there are no RDF elements on this XmlNode</returns>
		private RdfNode ParseRdfSyntax(XmlNode node, string nodeID)
		{
			//first get the NameSpace URI, NameSpace prefix and the LocalName for the node
			String nameSpaceURI = node.NamespaceURI;
			String nameSpacePrefix = node.Prefix;
			String localName = node.LocalName;
			RdfNode rNode = null;
			//known elements in the RDF Syntax are rdf:Description, rdf:Bag, rdf:Seq, rdf:Alt
			if(nameSpaceURI==RdfNamespaceCollection.RdfNamespace)
			{
				if(localName=="Description")
				{
					rNode = AddNodeToGraph(nodeID);
				}
				else if(localName=="Bag")
				{
					//its an rdf:Bag
					rNode = AddBagToGraph(nodeID);
				}
				else if(localName=="Seq")
				{
					//its an rdf:Seq
					rNode = AddSeqToGraph(nodeID);
				}
				else if(localName=="Alt")
				{
					//its an rdf:Alt
					rNode = AddAltToGraph(nodeID);
				}
			}
			return rNode;
		}
	}
}
