//
// backend-addressbook.cs: Evolution addressbook backend for the
// Dashboard.
//
// Nat Friedman <nat@nat.org>
//

using System;
using System.IO;
using System.Collections;
using System.Xml;
using GLib;
using System.Runtime.InteropServices;

namespace Dashboard {

	public class Addressbook : Backend {

		[DllImport ("libebook")]
		extern static IntPtr e_book_simple_find (string field, string needle, string how);

		Hashtable CardCache = new Hashtable ();

		public override bool Startup ()
		{
			Name = "Addressbook";

			try {
				IntPtr p = e_book_simple_find ("aim_name", "natfriedman", "is");
			} catch (System.MissingMethodException e) {
				Console.WriteLine ("Unable to load addressbook.  Make sure you have" +
						   "a patched Evolution installed.");
				return false;
			}

			this.SubscribeToClues ("full_name", 
					       "email", 
					       "aim_name", 
					       "yahoo_name", 
					       "jabber_name", 
					       "msn_name", 
					       "icq_name", 
					       "org", 
					       "url", 
					       "phone", 
					       "address", 
					       "rss", 
					       "keyword");
			this.Initialized = true;

			return true;
		}

		public override BackendResult ProcessCluePacket (CluePacket cp)
		{
			ArrayList matches  = new ArrayList ();
			ArrayList newclues = new ArrayList ();

			foreach (Clue c in cp.Clues)
				this.ProcessClue (c, matches, newclues);

			return new BackendResult (matches, newclues);
		}

		private void ProcessClue (Clue c, ArrayList matches, ArrayList newclues)
		{
			if (! ClueTypeSubscribed (c))
				return;

			if (c.Text == null || c.Text.Length < 3)
				return;

			// Find matching cards
			ArrayList cards = this.FindCards (c);
			if (cards.Count == 0)
				return;

			matches.Add (new Match (this, this.GenerateAllHtml (cards), 0, "addressbook card", c));

			// Generate new clues for chaining
			ArrayList clue_newclues = GenerateNewClues (cards, c);
			if (clue_newclues != null)
				newclues.AddRange (clue_newclues);
		}

		private ArrayList GenerateNewClues (ArrayList matches, Clue c)
		{
			ArrayList clues = new ArrayList ();

			if (matches.Count == 0)
				return null;

			foreach (string xml in matches) {
				XmlDocument card = new XmlDocument ();

				card.LoadXml (xml);

				string name  = GetXmlText (card, "//fname");
				string email = GetXmlText (card, "//email");
				string aim   = GetXmlText (card, "//aim");
				string url   = GetXmlText (card, "//url");
				string phone = GetXmlText (card, "//number");
				string rss   = GetXmlText (card, "//rss");

				clues.Add (new Clue ("full_name", name,  10, c));
				clues.Add (new Clue ("email",     email, 10, c));
				clues.Add (new Clue ("aim_name",  aim,   10, c));
				clues.Add (new Clue ("url",       url,   10, c));
				clues.Add (new Clue ("phone",     phone, 10, c));
				clues.Add (new Clue ("rss",       rss,   10, c));

				return clues;
			}

			return null;
		}

		private ArrayList FindCards (Clue clue)
		{
			IntPtr p;

			Console.WriteLine ("Addressbook backend finding: [{0}] [{1}] [{2}]",
					   clue.Type, clue.Text, "is");

			// Check the cache.
			lock (CardCache) {
				ArrayList cachematches = null;

				try {
					cachematches = (ArrayList) this.CardCache [clue.GetClueHash ()];
					if (cachematches != null) {
						Console.WriteLine ("Matches ARE in cache");
						return cachematches;
					}
				} catch {
					Console.WriteLine ("Matches NOT in cache");
				}

				// Look up the cards in the Evolution addressbook.
				lock (typeof (Addressbook)) {
					p = e_book_simple_find (clue.Type, clue.Text, "is");
				}

				GLib.List l = new GLib.List (p, typeof (string));

				ArrayList matches = new ArrayList (l);

				// Stick them in the cache.
				CardCache.Add (clue.GetClueHash (), matches);

				return matches;
			}
		}

		private string GetXmlText (XmlDocument xml, string tag)
		{
			XmlNode node;

			node = xml.SelectSingleNode (tag);
			if (node == null)
				return "";

			return node.InnerText;
		}

		private string GenerateAllHtml (ArrayList cards)
		{
			String html = "";

			foreach (string card in cards) 
				html += this.GenerateOneHtml (card);

			return html;
		}

		private string GenerateOneHtml (string xml)
		{
			XmlDocument card = new XmlDocument ();

			card.LoadXml (xml);

			string name  = GetXmlText (card, "//fname");
			string email = GetXmlText (card, "//email");
			string aim   = GetXmlText (card, "//aim");
			string url   = GetXmlText (card, "//url");
			string phone = GetXmlText (card, "//number");

			string html;

			html = GenerateHeaderHtml (name);
			html += GenerateEmailHtml (email);
			html += GenerateAimHtml (aim);
			html += GeneratePhoneHtml (phone);
			html += GenerateUrlHtml (url);
			html += GenerateFooterHtml ();

			return html;
		}

		private string GenerateHeaderHtml (string name)
		{
			return String.Format (
"      <table border=\"0\" height=\"100%\" width=100% valign=\"center\"> <tr> <td>" +
"	    <table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" align=\"center\" width=\"100%\">" +
"	      <tr bgcolor=\"#000000\"> <td>" +
"		  <table border=\"0\" cellspacing=\"1\" cellpadding=\"0\" width=\"100%\">" +
"		    <tr bgcolor=\"#f0f0f0\"> <td>" +
"			<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\">" +
"			  <tr bgcolor=\"#303030\"> " +
"                             <td valign=\"center\" align=\"left\">" +
"			          <img valign=center src=\"internal:person.png\">" +
"			      </td>" +
"                             <td valign=\"center\" align=\"left\" width=100%>" +
"			          <font color=#ffffff size=+2>{0}</font>" + 
"			      </td>" +
"			  </tr>" +
"			</table>" +
"<table border=0 width=100%>",
name);
		}

		private string GenerateEmailHtml (string email)
		{
			if (email.Length == 0)
				return "";

			return String.Format (
					      "<tr><td align=right>Email:</td> <td><tt><a href=\"mailto:{0}\">{0}</a></tt></td></tr>",
					      email);
		}

		private string GenerateAimHtml (string aim)
		{
			if (aim.Length == 0)
				return "";

			return String.Format (
					      "<tr><td align=right>AIM:</td> <td><tt><font color=\"#ccb933\">{0}</font></tt></td></tr>",
					      aim);
		}

		private string GeneratePhoneHtml (string phone)
		{
			if (phone.Length == 0)
				return "";

			return String.Format (
					      "<tr><td align=right>Phone:</td> <td><tt>{0}</tt></td></tr>",
					      phone);
		}

		private string GenerateUrlHtml (string url)
		{
			if (url.Length == 0)
				return "";

			return String.Format (
					      "<tr><td align=right>URL:</td> <td><tt><a href=\"{0}\">{0}</a></tt></td></tr>",
					      url);
		}

		private string GenerateFooterHtml ()
		{
			return String.Format (
"</table>" +
"</td>" +
"</tr>" +
"</table>" +
"		      </td> </tr>" +
"		  </table>" +
"		</td> </tr>" +
"	    </table>" +
	"      </table>");
		}


	} // class Addressbook

} // namespace Dashboard
