// Geosites backend: generates web links to related geographical
// resources
// by Edd Dumbill <edd@usefulinc.com>

using System;
using System.IO;
using System.Net;
using System.Web;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Threading;
using System.Runtime.Remoting.Messaging;
using System.Reflection;
using System.Runtime.InteropServices;

using Dashboard.Geo;

namespace Dashboard {

	class GeoSites : Backend {

		private Gazetteer gazetteer;

		public override bool Startup ()
		{
			Console.WriteLine ("GeoSites backend starting");

			this.SubscribeToClues ("latlong");
			this.Initialized = true;

			gazetteer = new Gazetteer ();

			Name = "GeoSites";

			return true;
		}

		public Match GetLinks (string txt)
		{
			string [] coords = Regex.Split(txt, "\\s*,\\s*");
			StringBuilder output = new StringBuilder ();

			if (coords.Length != 2)
				return null;

			string lat = coords[0];
			string lng = coords[1];

			double dlat = Double.Parse (lat);
			double dlng = Double.Parse (lng);

			int intlat = (int) dlat;
			int intlng = (int) dlng;

			output.Append (Header ("Local information"));



			Point p = gazetteer.NearestCity (dlat, dlng);

			if (p != null) {
				output.Append (String.Format ("<p><b>{0}, {1}, {2}</b></p>\n",
						p.City, p.Adminregion, p.Country));
			}
			
			output.Append ("<div>\n");

			output.Append ("<b>Maps</b>\n<ul>\n");

			output.Append (String.Format ("<li><a href=\"http://www.vicinity.com/myblast/map.mb?CMD=LFILL&CT={0}:{1}:90000\">MapBlast</a></li>\n", lat, lng));

			if (p != null) {
				String country = p.Country;

				// yawn, special case for mapquest
				if (country == "United States of America")
					country = "United States";

				// FIXME: MapQuest requires the state name as a two
				// letter code where the US is concerned.
				String mapquest = String.Format (
						"http://www.mapquest.com/maps/map.adp?city={0}&state={1}&country={2}&zoom=5",
						HttpUtility.UrlEncode (p.City),
						HttpUtility.UrlEncode (p.Adminregion),
						HttpUtility.UrlEncode (country));
				output.Append (String.Format ("<li><a href=\"{0}\">MapQuest</a></li>\n", mapquest));
			}
			
			output.Append ("</ul>\n");
			
			if (p != null) {
				output.Append ("<b>Weather</b>\n<ul>\n");
				String searchstring;

				// yawn, special case for wunderground
				// what is it with US-centricity? ;-p
				if (p.Country == "United States of America") {
					searchstring = String.Format ("{0}, {1}",
							p.City, p.Adminregion);
				} else {
					searchstring = String.Format ("{0}, {1}",
							p.City, p.Country);
				}

				output.Append (String.Format ("<li><a href=\"http://www.wunderground.com/cgi-bin/findweather/getForecast?query={0}\">wunderground.com</a></li>\n",
							HttpUtility.UrlEncode (searchstring)));


				output.Append ("</ul>\n");
			}
			
			output.Append ("<b>Images</b>\n<ul>\n");

			output.Append (String.Format ("<li><a href=\"http://www.confluence.org/confluence.php?lat={0}&lon={1}&visit=1\">Degree Confluence Project</a></li>\n", intlat, intlng));
			output.Append (String.Format ("<li><a href=\"http://terraserver.com/coordinates2.asp?y={0}&x={1}\">TerraServer</a></li>\n", lat, lng));

			output.Append ("</ul>\n");

			output.Append ("</div>\n");
			output.Append (Footer ());

			return new Match (this, output.ToString (), 0, "Local Info");
		}

		public override BackendResult ProcessCluePacket (CluePacket cp)
		{
			ArrayList matches = new ArrayList ();

			if (! this.Initialized)
				return null;

			foreach (Clue c in cp.Clues) {
				if (c.Text.Length == 0)
					continue;

				if (c.Type == "latlong") {
					Match m = GetLinks (c.Text);
					if (m != null)
						matches.Add (m);
				}
			}

			return new BackendResult (matches, null);
		}

		public static string Header (string title) {
			return String.Format (
					"      <table border=\"0\" height=\"100%\" width=\"100%\" valign=\"center\"> <tr> <td>" + 
            "       <table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" align=\"center\" width=\"100%\">" + 
            "         <tr bgcolor=\"#000000\"> <td>" + 
            "         <table border=\"0\" cellspacing=\"1\" cellpadding=\"0\" width=\"100%\">" + 
            "           <tr bgcolor=\"#f0f0f0\"> <td>" + 
            "           <table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\">" + 
            "             <tr bgcolor=\"#303030\"> " + 
            "                             <td valign=\"center\" align=\"left\">" + 
            "                     <img valign=\"center\" src=\"internal:globe.png\">" + 
            "                 </td>" + 
            "                             <td valign=\"center\" align=\"left\" width=100%>" + 
            "                     <font color=\"#ffffff\" size=\"+2\">{0}</font>"  + 
            "                 </td>" + 
            "             </tr>" + 
            "           </table>" + 
            "<table border=\"0\" width=\"100%\">",
			HttpUtility.HtmlEncode (title));
		}

		public static string Footer () 
		{
			return "</table></td></tr></table>" + 
                "</td></tr></table></td></tr></table>";
		}

	}

}

