//
// backend-rss.cs: A backend which displays recent entries from an RSS
// feed.
//
// Nat Friedman <nat@nat.org>
// Edd Dumbill <edd@usefulinc.com>
//

using System;
using System.IO;
using System.Collections;
using System.Xml;
using System.Runtime.InteropServices;
using System.Net;

namespace Dashboard {

	class RSS : BackendSimple {

		public override bool Startup ()
		{
			Name = "RSS";

			Console.WriteLine ("RSS Backend starting up");

			this.SubscribeToClues ("rss");
			this.Initialized = true;

			return true;
		}

		protected override Match ProcessClueSimple (Clue clue)
		{
			if (clue.Type != "rss")
				return null;

			XmlDocument xml = this.GetRSS (clue.Text);
			if (xml == null)
				return null;

			// Initialize namespaces for parsing an RSS 1.0 feed
			XmlNamespaceManager ns = 
				new XmlNamespaceManager(xml.NameTable);
			ns.AddNamespace ("rss", 
					"http://purl.org/rss/1.0/");
			ns.AddNamespace ("rdf", 
					"http://www.w3.org/1999/02/22-rdf-syntax-ns#");
			ns.AddNamespace ("dc", 
					"http://purl.org/dc/elements/1.1/");

			string title;
			string link;
			ArrayList items;

			// check whether we're processing RSS 1.0 or not
			if (xml.SelectSingleNode ("/rdf:RDF/rss:channel", ns) != null) {
				// yes, rdf:RDF/rss:channel is there
				Console.WriteLine ("Found an RSS 1.0 feed.");
				title = this.GetXmlNodeText (xml, 
						"/rdf:RDF/rss:channel/rss:title", ns);
				link  = this.GetXmlNodeText (xml,
						"/rdf:RDF/rss:channel/rss:link", ns);
				items = this.GetRecentRSSItems (xml, ns);
			} else {
				// no, we assume RSS 0.91 or 2.0
				Console.WriteLine ("Found an RSS 0.91/2.0 feed.");
				title = this.GetXmlNodeText (xml, "//title");
				link  = this.GetXmlNodeText (xml, "//link");
				items = this.GetRecentRSSItems (xml);
			}

			string html = this.GenerateAllHTML (items, title, link);

			return new Match (this, html, 0, String.Format ("Blog Entries:{0}", clue.Text));
		}

		private XmlDocument GetRSS (string url) {

			if (url == null || url.Length == 0)
				return null;

			HttpWebRequest req = (HttpWebRequest) WebRequest.Create (url);
			req.UserAgent = "GNOME Dashboard";
			WebResponse resp = req.GetResponse ();
			StreamReader input = new StreamReader 
				(resp.GetResponseStream ());

			string rss = input.ReadToEnd ();

			XmlDocument xml = new XmlDocument ();
			
			try {
				xml.LoadXml (rss);
			} catch (Exception e) {
				Console.WriteLine ("RSS Parsing Exception {0}", e);
				return null;
			}

			return xml;
		}

		private ArrayList GetRecentRSSItems (XmlDocument xml) {
			ArrayList items = new ArrayList ();
			XmlNodeList items_xml = xml.SelectNodes ("//item");
			foreach (XmlNode item_xml in items_xml) {
				string link    = this.GetXmlNodeText (item_xml, "link");
				string title   = this.GetXmlNodeText (item_xml, "title");
				string pubdate = this.GetXmlNodeText (item_xml, "pubDate");

				items.Add (new RSSItem (link, title, pubdate));
			}

			return items;
		}

		// RSS 1.0 specific way to get items
		private ArrayList GetRecentRSSItems (XmlDocument xml,
				XmlNamespaceManager ns) {

			ArrayList items = new ArrayList ();
			XmlNodeList items_xml = xml.SelectNodes ("/rdf:RDF/rss:item", ns);
			foreach (XmlNode item_xml in items_xml) {
				string link    = this.GetXmlNodeText (item_xml, "rss:link", ns);
				string title   = this.GetXmlNodeText (item_xml, "rss:title", ns);
				string pubdate = this.GetXmlNodeText (item_xml, "dc:date", ns);

				items.Add (new RSSItem (link, title, pubdate));
			}

			return items;
		}

		private string GenerateAllHTML (ArrayList items, string title, string link) {
			string html;

			html = String.Format (
"<table border=0 width=100%>" +
"<tr><td bgcolor=#000000 text=#ffffff>" +
"<a href=\"{0}\" style=\"text-decoration: none;\"><font color=#ffffff size=+2><b>{1} (blog)</b></font></a>" +
"</td></tr>", link, title);

			int count = 0;
			foreach (RSSItem item in items) {
				html += String.Format ("<tr><td><li><a href=\"{0}\">{1}</a><br></td></tr>",
						       item.Link, item.Title);
				count ++;
				if (count > 3)
					break;
			}

			html += "</table>";
			return html;
		}

		private string GetXmlNodeText (XmlNode xml, string tag,
				XmlNamespaceManager ns)
		{
			XmlNode node;

			node = xml.SelectSingleNode (tag, ns);
			if (node == null)
				return "";

			return node.InnerText;
		}

		private string GetXmlNodeText (XmlNode xml, string tag)
		{
			XmlNode node;

			node = xml.SelectSingleNode (tag);
			if (node == null)
				return "";

			return node.InnerText;
		}

	} // class RSS

	class RSSItem {
		public string Link;
		public string Title;
		public string PubDate;

		public RSSItem (string Link, string Title, string PubDate) {
			this.Link    = Link;
			this.Title   = Title;
			this.PubDate = PubDate;
		}
	}

} // namespace Dashboard
