//
// GNOME Dashboard
//
// backend-textchain.cs: Produces no matches, but extracts a few
// simple clues from textblocks and chains them into the engine.
//
// Author:
//    Nat Friedman <nat@nat.org>
//

using System;
using System.IO;
using System.Collections;
using System.Text.RegularExpressions;
using Gtk;

namespace Dashboard {

	class TextChain : Backend {

		private const string REGEX_URL
			= "((((http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?(/)?)";

		// this regex found at http://www.dotnetforums.net/t49640.html
		private const string REGEX_EMAIL
			= "([\\w\\.\\-]+@[a-zA-Z0-9\\-]+(\\.[a-zA-Z0-9\\-]{1,})*(\\.[a-zA-Z]{2,3}){1,2})";

		private const string REGEX_PHONE_US
			= "(\\(?[2-9][0-9]{2}\\)?[-. ][1-9][0-9]{2}[-. ][0-9]{4})";

		private const string REGEX_PHONE
			= "([\\+\\(]?\\d(\\d|\\)|[\\-\\(\\. ]\\d){6,})[\\s$]";

		private const string REGEX_BUGZILLA
			= "#([1-9][0-9]*)";
		
		public override bool Startup ()
		{
			Name = "TextChain";

			this.SubscribeToClues ("textblock", 
					       "htmlblock",
					       "content",
					       "sentence_at_point");
			this.Initialized = true;

			return true;
		}

		class TextBlockReceiver {
			public string Textblock = "";

			public bool ReceiveText (IntPtr obj, string data)
			{
				Textblock += data;
				return true;
			}
		}

		// convert htmlblocks to textblocks (if a translation does not
		// exist already) for allowing backends that don't know how to
		// handle html to process the text.
		void ConvertHtmlToText (CluePacket cp, ArrayList newclues)
		{
			ArrayList htmlblocks = cp.GetClues ("htmlblock");

			// remove textblocks already converted from htmlblocks
			// (as marked by the TriggeringClues field).
			foreach (Clue c in cp.GetClues ("textblock")) {
				if (c.TriggeringClues.Count == 1) {
					Clue src_c = (Clue) c.TriggeringClues [0];
					if (src_c.Type == "htmlblock")
						htmlblocks.Remove (src_c);
				}
			}

			foreach (Clue c in htmlblocks) {
				Gtk.HTML html2txt = new Gtk.HTML (c.Text);
				TextBlockReceiver recv = new TextBlockReceiver();

				html2txt.Export ("text/plain", new HTMLSaveReceiverFn (recv.ReceiveText));

				if (recv.Textblock.Length > 0) {
					newclues.Add (new Clue ("textblock", 
								recv.Textblock.TrimEnd ('\n'), 
								c.Relevance, 
								c));
				}
			}
		}

		public override BackendResult ProcessCluePacket (CluePacket cp)
		{
			ArrayList newclues = new ArrayList ();
			ArrayList urlclues = new ArrayList ();

			foreach (Clue clue in cp.Clues) {
				if (! ClueTypeSubscribed (clue))
					continue;

				// we normalize url clues to have their
				// URI schemes on the front
				
				urlclues.AddRange 
					(this.GetPatternMatches (clue, "url", REGEX_URL));

				foreach (Clue c in urlclues) {
					Clue r = null;

					if (String.Compare 
							(c.Text, 0, "www.", 0, 4, true) == 0) {
						r = new Clue (c.Type, "http://" + c.Text, c.Relevance, c);
					} else if (String.Compare 
							(c.Text, 0, "ftp.", 0, 4, true) == 0) {
						r = new Clue (c.Type, "ftp://" + c.Text, c.Relevance, c);
					}
					if (r == null) {
						newclues.Add (c);
					} else {
						newclues.Add (r);
					}
				}

				newclues.AddRange (this.GetPatternMatches (clue, "email",    REGEX_EMAIL));
				newclues.AddRange (this.GetPatternMatches (clue, "phone",    REGEX_PHONE));
				newclues.AddRange (this.GetPatternMatches (clue, "bugzilla", REGEX_BUGZILLA));
			}

			// convert htmlblocks to textblocks
			ConvertHtmlToText (cp, newclues);

			if (newclues.Count == 0)
				newclues = null;

			return new BackendResult (null, newclues);
		}

		private ArrayList GetPatternMatches (Clue clue, string type, string regex)
		{
			Regex r;
			System.Text.RegularExpressions.Match m;

			r = new Regex (regex, RegexOptions.IgnoreCase | RegexOptions.Compiled);

			ArrayList newclues = new ArrayList ();
			for (m = r.Match (clue.Text); m.Success; m = m.NextMatch ())
				newclues.Add (new Clue (type, m.Groups[1].ToString (), 10, clue));

			return newclues;
		}
	}
}
