/**
 *  file indexer.
 *  creates an index of files based on path and LastWriteTime.
 */

using System;
using System.IO;
using System.Text;
using System.Collections;

namespace FileIndex {

public class Node {
	public  const int FIELD_ARRAY_LENGTH = 1024;

	private const int PATH_OFFSET        = 8;

	private string Path;
	private long LastWriteTime;

	public Node(FileInfo file) {
		Path = file.FullName;
		LastWriteTime = file.LastWriteTime.ToFileTime();
	}

/*
 * first 8 bytes is the LastWriteTime of the file,
 * remaining bytes is the file path,
 * zero padded to complete the 1024 byte node
 */

	public Node(byte[] field_array) {
		LastWriteTime = BitConverter.ToInt64(field_array, 0);

		int len = 0;
		while (field_array[len + PATH_OFFSET] != 0)
			++len;

		char[] path_chars = new char[len];
		Encoding.ASCII.GetChars(field_array, PATH_OFFSET, len, path_chars, 0);

		Path = new string(path_chars);
	}

	public byte[] ToFieldArray() {
		byte[] field_array = new byte[FIELD_ARRAY_LENGTH];

		Array.Copy(BitConverter.GetBytes(LastWriteTime), 0, field_array, 0, 8);
		Array.Copy(Encoding.ASCII.GetBytes(Path), 0, field_array, PATH_OFFSET, Path.Length);

	// pad with zeroes

		for (int i=PATH_OFFSET + Path.Length; i<FIELD_ARRAY_LENGTH; ++i)
			field_array[i] = 0;

		return field_array;
	}

	public string GetPath() {
		return Path;
	}

	public long GetLastWriteTime() {
		return LastWriteTime;
	}

	public bool MatchPath(string pat) {
		return Path.IndexOf(pat) >= 0;
	}

	public bool MatchTime(DateTime time, long limit) {
		return Math.Abs(time.ToFileTime() - LastWriteTime) < limit;
	}

	public override string ToString() {
		return String.Format("{0} [{1}]", Path, LastWriteTime);
	}
}

public class FileIndex {
	private const string INDEX_PATH = "/home/jimmyk/dashboard/file_index/files.idx";

	private static FileIndex instance = null;

	private FileStream fs;

	public FileIndex() {
		fs = new FileStream(INDEX_PATH, FileMode.OpenOrCreate, FileAccess.ReadWrite);
	}

	public static FileIndex GetInstance() {
		if (instance == null)
			instance = new FileIndex();

		return instance;
	}

/*
 * read the next FIELD_ARRAY_LENGTH bytes from the stream and create a Node
 * object
 */

	private Node PopNode() {
		byte[] field_array = new byte[Node.FIELD_ARRAY_LENGTH];

		if (fs.Read(field_array, 0, field_array.Length) >= field_array.Length)
			return new Node(field_array);

		return null;
	}

/*
 * rewind the file
 */

	private void ResetFileStream() {
		fs.Close();
		fs = new FileStream(INDEX_PATH, FileMode.OpenOrCreate, FileAccess.ReadWrite);
	}

/*
 * position the cursor to the location in the index which holds the node
 * specified by the given path.  it reads the file until it finds the node,
 * counting as it goes, rewinds the file and reads just up to the node.  in the
 * event that no such path exists the function returns with the cursor placed
 * at the end of the file.
 */

	private void PositionCursor(string path) {
		ResetFileStream();

		int cursor = 0;
		for (Node n = PopNode(); n != null; n = PopNode(), ++cursor) {
			if (n.GetPath().Equals(path)) {
				ResetFileStream();

				for (int i=0; i<cursor; ++i)
					PopNode();

				return;
			}
		}
	}

/*
 * find all nodes which contain the string pat in the path.
 */

	public ArrayList FindNodes(string pat) {
		ArrayList nodes = new ArrayList();

		for (Node n = PopNode(); n != null; n = PopNode())
			if (n.MatchPath(pat))
				nodes.Add(n);

		ResetFileStream();

		return nodes;
	}

/*
 * find all nodes where the LastWriteTime is within limit ms of the
 * given time.
 */

	public ArrayList FindNodes(DateTime time, long limit) {
		ArrayList nodes = new ArrayList();

		for (Node n = PopNode(); n != null; n = PopNode())
			if (n.MatchTime(time, limit))
				nodes.Add(n);

		ResetFileStream();

		return nodes;
	}

/*
 * update a file in the index, in the event that the file doesn't exist, add
 * it.  in the even that the given file is a directory, recursively add all
 * subfiles, dirs.
 */

	public void UpdateIndex(string path) {
		if (Directory.Exists(path)) {
			foreach (string child in Directory.GetFiles(path))
				UpdateIndex(child);

			foreach (string child in Directory.GetDirectories(path))
				UpdateIndex(child);
		}
		else {
			PositionCursor(path);

			Node n = new Node(new FileInfo(path));

			byte[] data = n.ToFieldArray();

			fs.Write(data, 0, data.Length);

			ResetFileStream();
		}
	}

	public void OutputIndex() {
		int line = 0;
		for (Node n = PopNode(); n != null; n = PopNode(), ++line)
			Console.WriteLine(line + ": " + n);

		ResetFileStream();
	}
}

public class TestHarness {
	public static void Main(string[] args) {
		FileIndex index = FileIndex.GetInstance();

//		index.UpdateIndex(args[0]);

		index.OutputIndex();

		FileInfo file = new FileInfo(args[0]);
		long limit = Int64.Parse(args[1]);

		ArrayList nodes = index.FindNodes(file.LastWriteTime, limit);

		IEnumerator iter = nodes.GetEnumerator();
		while (iter.MoveNext()) {
			Node n = (Node)iter.Current;

			Console.WriteLine(n.GetPath());
		}
	}
}

}
