//
// GNOME Dashboard
//
// backend.cs: Base class for dashboard backends.
//
// Authors:
//    Nat Friedman <nat@nat.org>
//    Joe Shaw <joe@assbarn.com>
//

using System;
using System.Collections;

namespace Dashboard {

	public abstract class Backend {
		// All backends must implement a 'startup' method which
		// gets called when the dashboard starts and loads them.
		//
		// When this function is finished, the backend must set
		// this.Initialized to true before any queries will be
		// sent to it.
		public abstract bool  Startup ();

		public IList SubscribedClues = null;

		protected void SubscribeToClues (params string[] clue_types_supported) 
		{
			SubscribedClues = clue_types_supported;
		}

		protected bool ClueTypeSubscribed (Clue c)
		{
			if (SubscribedClues == null)
				return true;

			if (c.Type == "*")
				return true;

			return SubscribedClues.Contains (c.Type);
		}

		private CluePacket LastCluePacket = null;
		private int LastSubscriptionHash = 0;

		public virtual bool AcceptCluePacket (CluePacket cp)
		{
			// If the backend hasn't specified the clue_types its
			// interested in, always accept the CluePacket.
			if (SubscribedClues == null)
				return true;

			if (this.LastCluePacket == null || 
			    this.LastCluePacket.IsParent (cp) == false) {
				this.LastCluePacket = cp;
				this.LastSubscriptionHash = 0;
			}

			int subscription_hash = 0;

			foreach (Clue clue in cp.Clues) {
				if (this.SubscribedClues.Contains (clue.Type))
					subscription_hash += clue.GetClueHash ();
			}

			if (subscription_hash != 0 && 
			    subscription_hash != this.LastSubscriptionHash) {
				this.LastSubscriptionHash = subscription_hash;
				Console.WriteLine ("Accepting Packet: '{0}'", this.Name);
				return true;
			} else {
				Console.WriteLine ("Denying Packet: '{0}'", this.Name);
				return false;
			}
		}

		// This is the core backend function which processes a
		// cluepacket to:
		//     - search for matches
		//     - generate new clues
		//     - anything else it wants to do, like logging
		//       cluepackets to a file
		public abstract BackendResult ProcessCluePacket (CluePacket cp);

		// To handle clicks for non-URLs and non-local-paths.  
		// Default implementation does nothing.
		public virtual void LinkClicked (string user_data)
		{
		}

		// The backend's name.
		public string Name;

		// Whether or not the backend is ready to accept clues.
		public bool Initialized;

		public override string ToString ()
		{
			return Name;
		}
	}

	public abstract class BackendSimple : Backend {
		// This implementation calls ProcessClueSimple for each clue in
		// a cluepacket.  ProcessSimpleClue must be implemented by
		// subclasses.
		public override BackendResult ProcessCluePacket (CluePacket cp)
		{
			ArrayList matches = new ArrayList ();

			foreach (Clue c in cp.Clues) {
				if (ClueTypeSubscribed (c)) {
					Match m = this.ProcessClueSimple (c);
					if (m != null && m.Html != null) {
						m.TriggeringClues.Add (c);
						matches.Add (m);
					}
				}
			}

			return new BackendResult (matches, null);
		}

		protected abstract Match ProcessClueSimple (Clue c);
	}
}
