//
// GNOME Dashboard
//
// dashboard.cs: Startup, some GUI code and some control flow.  A
// little messy.
//
// Authors:
//   Nat Friedman <nat@nat.org>
//   Joe Shaw <joe@assbarn.com>
//   Miguel de Icaza <miguel@ximian.com>
//

using System;
using System.Collections;
using System.IO;
using System.Threading;

using Gtk;
using Gnome;
using GtkSharp;

namespace Dashboard {
	public delegate void GuiMatchNotifyDelegate (Match m);
	public delegate void GuiClearDelegate ();
	
	class Driver {
		static void Main (string [] args)
		{
			Program dboard = new Program ("Dashboard", "0.0", Modules.UI, args);
			Thread.CurrentThread.Name = "Main thread";

			Dashboard d = new Dashboard ();

			dboard.Run ();
		}
	}

	class Dashboard {
		private CluePacketManager cpm;
		private MatchFilter       filter;
		private string            dashdir;
		private Window            window;
    		private TcpServer         server;
		
		public Dashboard ()
		{
			Console.WriteLine ("Dashboard starting up");

			Gnome.VFS.Mime.Init ();

			// Create directories we need
			SetupDirectories ();

			// Create the MatchFilter and CluePacket Manager
			filter = new MatchFilter (new GuiMatchNotifyDelegate (GuiMatchNotifyCallback),
						  new GuiClearDelegate       (GuiClearCallback));
			cpm    = new CluePacketManager (filter);
			cpm.LoadBackends ();

			// Start the TCP server.
			StartupServer ();

			// Create the window.
			SetupGUI ();

		}

		//
		// Invoked by the MatchFilter with a new match to draw.
		//
		private void GuiMatchNotifyCallback (Match m)
		{
			Console.WriteLine ("GuiMatchNotifyCallback got match");

			this.window.Display (m);
		}

		//
		// Invoked by the MatchFilter to clear the display.
		//
		private void GuiClearCallback ()
		{
			Console.WriteLine ("GuiClearCallback got invoked");

			this.window.Clear ();
		}

		//
		// Invoked by the server when it gets a new cluepacket
		// from some frontend.
		//
		private void CluePacketNotifyCallback (CluePacket cp)
		{
			Console.WriteLine ("Got a new cluepacket");

			// Replace the entry text with the new clues
			string text = "";
			foreach (Clue c in cp.Clues) {
				if (text != "") {
					text += ", ";
				}

				text += c.Text;
			}

			this.window.QueryEntry.Text = text;

			// Find matches.
			cpm.ProcessFrontendCluePacket (cp);
		}

		private void SetupDirectories ()
		{
			string home_dir = Environment.GetEnvironmentVariable ("HOME");
			string path = Path.Combine (home_dir, ".dashboard");

			dashdir = path;

			try {
				Directory.CreateDirectory (path);
			} catch {
				return;
			}

			path = Path.Combine (path, "tmp");
			try {
				Directory.CreateDirectory (path);
			} catch {
				return;
			}
		}

		//
		// Startup code
		//
		private void StartupServer ()
		{
			server = new TcpServer ();
			server.CluePacketNotify += new CluePacketNotifyDelegate (CluePacketNotifyCallback);
			Thread t = new Thread (new ThreadStart (server.Run));
			t.Name = "Dashboard Server";
			t.Start ();
		}

		private void SetupGUI ()
		{
			Console.WriteLine ("Setting up GUI...");

			this.window = new Window ("Dashboard", this.cpm);
			this.window.DeleteEvent += new DeleteEventHandler (WindowDelete);

			EventHandler handler = new EventHandler (SendQuery);

			this.window.QueryEntry.Activated += handler;
			this.window.QueryButton.Clicked += handler;

			EventHandler graph_handler = new EventHandler (ShowGraph);
			this.window.GraphButton.Clicked += graph_handler;

			this.window.ShowAll ();
		}

		private void WindowDelete (object o, DeleteEventArgs args)
		{
			Application.Quit ();
			server.Stop ();
			args.RetVal = true;
		}

		private void SendQuery (object o, EventArgs e) {
			CluePacket cp = new CluePacket ("Dashboard", "Default", true);
			cp.Clues.Add (new Clue ("*", this.window.QueryEntry.Text, 10));

			this.cpm.ProcessFrontendCluePacket (cp);
		}

		private void ShowGraph (object o, EventArgs e) {
			filter.CreateClueChainGraph ();
		}
	}
}
