//
// GNOME Dashboard
//
// gui.cs: User interface routines
//
// Authors:
//    Nat Friedman <nat@nat.org>
//    Joe Shaw <joe@assbarn.com>
//    Miguel de Icaza <miguel@ximian.com>
//

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;

using Gtk;
using Gnome;
using GtkSharp;

namespace Dashboard {

	public class Window : Gtk.Window {

		private Gtk.Entry      entry;
		private Gtk.Button     button;
		private Gtk.Button     graph_button;
		private Gtk.HTML       html;
		private Gtk.HTMLStream stream;

		private Queue          match_queue;
		private bool           ClearAtNextMatch = true;
		private ThreadNotify   notify_gtk;

		private CluePacketManager cpm;

		public Window (string title, CluePacketManager cpm) : base (WindowType.Toplevel)
		{
			this.Title         = title;
			this.cpm           = cpm;
			this.DefaultWidth  = 279;
			this.DefaultHeight = 640;

			Box hbox = new HBox (false, 0);

			this.entry = new Gtk.Entry ();
			hbox.PackStart (entry, false, false, 2);
			
			this.button = new Button ("Query");
			hbox.PackStart (button, false, false, 2);

			this.graph_button = new Button ("Graph");
			hbox.PackStart (graph_button, false, false, 2);

			this.html = new Gtk.HTML ();
			this.html.LinkClicked += new LinkClickedHandler (LinkClicked);
			this.html.UrlRequested += new UrlRequestedHandler (UrlRequested);

			ScrolledWindow view = new ScrolledWindow ();
			view.SetPolicy (PolicyType.Never, PolicyType.Automatic);
			view.Add (this.html);
			
			Box vbox = new VBox (false, 0);
			vbox.PackStart (hbox, false, false, 0);
			vbox.PackStart (view, true, true, 0);

			this.Add (vbox);
		      
			this.match_queue = new Queue ();
			this.notify_gtk = new ThreadNotify (new ReadyEvent (ProcessMatchQueue));
		}

		private void LinkClicked (object o, LinkClickedArgs args)
		{
			// If it's backend-specific, call into the
			// backend.
			if (args.Url.StartsWith ("backend:")) {
				// FIXME: The CPM should not keep the backend list.
				this.cpm.NotifyBackendLinkClicked (args.Url);
				return;
			} else if (args.Url.StartsWith ("exec:")) {
				string cmd = args.Url.Substring ("exec:".Length);

				Process p = new Process ();
				p.StartInfo.UseShellExecute = false;
				if (cmd.IndexOf (' ') == -1)
					p.StartInfo.FileName = cmd;
				else {
					p.StartInfo.FileName = cmd.Substring (0, cmd.IndexOf (' '));
					p.StartInfo.Arguments = cmd.Substring (cmd.IndexOf (' '));
				}

				try {
					p.Start ();
				} catch {
				}

				return;
			}

			// Otherwise, open with the proper MIME
			// handler.
			Gnome.MimeApplication.Exec (Gnome.VFS.Mime.GetMimeType (args.Url), args.Url);			
		}

		private void OpenFileWithUserApp (string path)
		{
			
		}
			
		private Stream GetImage (string name)
		{
			Assembly assembly = System.Reflection.Assembly.GetCallingAssembly ();
			System.IO.Stream s = assembly.GetManifestResourceStream (name);

			return s;
		}

		private Stream GetFile (string uri)
		{
			string path = uri;
			Stream s = File.Open (path, FileMode.Open, FileAccess.Read);

			return s;
		}

		//
		// Provides data for urls requested (images).  Things prefixed
		// with `internal:' we pull for one of the embedded streams
		//
		private void UrlRequested (object o, UrlRequestedArgs args)
		{
			Console.WriteLine ("HTML widget requesting: " + args.Url);
			Stream s = null;

			if (args.Url.IndexOf ("/") == 0) {
				s = GetFile (args.Url);
			} else {
				if (args.Url.StartsWith ("internal:")) {
					try {
						s = GetImage (args.Url.Substring (args.Url.IndexOf (':') + 1));
					} catch {
						Console.WriteLine ("Could not find image: " + args.Url);
						return;
					}
				} else {
					Console.WriteLine ("Do not know how to handle " + args.Url);
					return;
				}
			}

			byte [] buffer = new byte [8192];
			int n;
			
			while ((n = s.Read (buffer, 0, 8192)) != 0) {
				args.Handle.Write (buffer, n);
			}
		}

		// Called in the main loop (in the main thread) when threads
		// have queued work for us.
		private void ProcessMatchQueue () {
			lock (this.match_queue) {
				while (this.match_queue.Count > 0) {
					Match m = (Match) this.match_queue.Dequeue ();

					if (this.ClearAtNextMatch) {
						Console.WriteLine ("Clearing Display");
						this.stream = this.html.Begin();
						this.stream.Write ("<html><body>");

						this.ClearAtNextMatch = false;
					}

					this.stream.Write (m.Html);
				}
 			}
		}

		public void Clear ()
		{
			this.ClearAtNextMatch = true;
		}

		public void Display (Match m)
		{
			lock (this.match_queue) {
				this.match_queue.Enqueue (m);
			}
			notify_gtk.WakeupMain ();
		}

		public Gtk.Entry QueryEntry
		{
			get {
				return this.entry;
			}
		}

		public Gtk.Button QueryButton
		{
			get {
				return this.button;
			}
		}

		public Gtk.Button GraphButton
		{
			get {
				return this.graph_button;
			}
		}
	}
}



