//
// match-filter.cs: Implement the MatchFilter, which receives matches
// from the CluePacketManager as they come in from backends, filters
// out irrelevant ones (like matches from cluepackets that are no
// longer current), sorts them, and sends them to the GUI.
//
// Author:
//    Nat Friedman <nat@nat.org>

using System;
using System.Collections;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Diagnostics;

namespace Dashboard {

	public class MatchFilter {

		// This is the function we call to ask the GUI to draw
		// a new match.
		private GuiMatchNotifyDelegate GuiMatchNotify;

		// This is the function we use to clear the display.
		private GuiClearDelegate GuiClear;

		// The most-current query that the dashboard has
		// received from a frontend.  Matches which come in
		// for older queries are discarded.
		private CluePacket LatestCluePacket;

		// The query that's currently being displayed.
		// This is distinct from the latest one, since we
		// leave the matches for the currently-displayed
		// query up until the first match comes in
		// for LatestContext.
		private CluePacket CurrentlyDisplayedCluePacket;

		// All the currently-displayed matches are listed here,
		// so we can avoid match duplication in the GUI.
		private ArrayList DisplayedMatches = new ArrayList ();

		// All the matches we've received for the
		// currently-displayed matchset, including duplicates.
		// For debugging purposes.
		private ArrayList AllMatches = new ArrayList ();

		public MatchFilter (GuiMatchNotifyDelegate gui_match_notify,
				    GuiClearDelegate       gui_clear)
		{
			this.GuiMatchNotify = gui_match_notify;
			this.GuiClear       = gui_clear;
		}

		// The CluePacketManager calls this when a new cluepacket
		// comes in from a frontend.
		public void NotifyNewCluePacket (CluePacket cp)
		{
			if (! cp.Focused)
				return;

			this.LatestCluePacket = cp;
		}

		// CluePacketManager calls this to let us know about a new
		// match that comes in from a backend.
		public void NotifyMatch (CluePacket cp, Match m)
		{
			lock (this) {
				if (! cp.Focused)
					return;
				
				// Ignore matches from old cluepackets.
				if (this.LatestCluePacket != cp &&
				    (this.LatestCluePacket == null ||
				     ! this.LatestCluePacket.IsParent (cp)))
					return;

				if (this.CurrentlyDisplayedCluePacket == null || 
				    ! this.CurrentlyDisplayedCluePacket.IsParent (cp)) {
					this.GuiClear ();
					this.DisplayedMatches = new ArrayList ();
					this.AllMatches = new ArrayList ();
					this.CurrentlyDisplayedCluePacket = cp;
				}

				Console.WriteLine ("Got a match: " + m.Desc);

				this.AllMatches.Add (m);

				// Cull duplicate matches.
				foreach (Match displayed_match in this.DisplayedMatches)
					if (displayed_match.Html == m.Html)
						return;

				// Stick the match in the already-displayed list.
				this.DisplayedMatches.Add (m);

				// Send the match to the GUI to be displayed.
				this.GuiMatchNotify (m);
			}
		}

		// This pops up an EOG window with a graph of the
		// cluechain.  The GUI calls it when the user hits the
		// 'cluechain graph' button.
		//
		// Requires that you have dot installed. 
		//
		// Just for fun.
		//
		public void CreateClueChainGraph ()
		{
			string home = Environment.GetEnvironmentVariable ("HOME");
			string path = Path.Combine (home, ".dashboard/tmp/cluegraph.dot");

			FileStream s;

			try {
				s = File.Open (path, FileMode.Create);
			} catch {
				return;
			}

			StreamWriter sw = new StreamWriter (s);

			// Write the header.
			sw.WriteLine ("digraph \"cluechain\" {");
			sw.WriteLine ();

			CluePacket frontend_cp = this.CurrentlyDisplayedCluePacket;
			while (frontend_cp.Parent != null)
				frontend_cp = frontend_cp.Parent;

			sw.WriteLine ("    frontend_cluepacket [ label=\"Original Cluepacket from {0}\" " +
				      "color=\"green3\" shape=\"box\" style=\"filled\"];",
				      frontend_cp.Frontend);

			foreach (Clue c in frontend_cp.Clues) 
				sw.WriteLine ("    frontend_cluepacket -> {0};", c.ToShortOpaqueString ());

			foreach (Match m in this.AllMatches) {
				// The Match
				sw.WriteLine (this.GraphMatchString (m));

				// The links from the clues to the match
				foreach (Clue c in m.TriggeringClues)
					sw.WriteLine ("    {0} -> {1};", c.ToShortOpaqueString (), m.ToShortOpaqueString ());
			}

			foreach (Clue c in this.CurrentlyDisplayedCluePacket.Clues) {
				sw.WriteLine (this.GraphClueString (c));

				foreach (Clue pc in c.TriggeringClues)
					sw.WriteLine ("    {0} -> {1} [style=\"dashed\"];",
						      pc.ToShortOpaqueString (), c.ToShortOpaqueString ());
			}

			sw.WriteLine ("}");
			sw.Flush ();
			sw.Close ();

			string pngpath = Path.Combine (home, ".dashboard/tmp/cluegraph.png");
			// Run dot to convert this to a .png.
			Process dot = new Process ();
			dot.StartInfo.FileName = "dot";
			dot.StartInfo.Arguments = String.Format ("-Tpng {0} -o {1}", path, pngpath);
			dot.StartInfo.UseShellExecute = true;

			try {
				dot.Start ();
			} catch {
				return;
			}

			// Run EOG to display it.
			Process eog = new Process ();
			eog.StartInfo.FileName = "gthumb";
			eog.StartInfo.Arguments = pngpath;
			eog.StartInfo.UseShellExecute = true;
			try {
				eog.Start ();
			} catch {
				return;
			}
		}

		private string GraphClueString (Clue c)
		{
			string cluetext = c.Text.Substring (0, Math.Min (12, c.Text.Length));
			cluetext = cluetext.Replace ("\"", "");
			cluetext = cluetext.Replace ("\n", "");
			cluetext = cluetext.Replace ("[",  "");
			cluetext = cluetext.Replace ("]",  "");

			return String.Format
				("    {0} [ label=\"Clue Type: {1} Text: {2}\" color=\"dodgerblue\" style=\"filled\"];",
				 c.ToShortOpaqueString (), c.Type, cluetext);
		}

		private string GraphMatchString (Match m)
		{
			return String.Format
				("    {0} [ label=\"Match: {1}\" color=\"indianred\" shape=\"box\" style=\"filled\"];",
				 m.ToShortOpaqueString (), m.Desc);
		}

	}
}
