#include "internal.h"

#include "gtkconv.h"
#include "conversation.h"
#include "gtkimhtml.h"
#include "gtkplugin.h"
#include "debug.h"

#include "dashboard-frontend.c"

#define PLUGIN_ID "dashboard"

static gboolean
gaim_gtkconv_is_focused (GaimGtkConversation *gtkconv)
{

	if (GTK_WIDGET_HAS_FOCUS (gtkconv->imhtml) ||
	    GTK_WIDGET_HAS_FOCUS (gtkconv->entry))
		return TRUE;

	return FALSE;
}

static char *
strip_uninteresting (char *text)
{
	char *new;
	char *srcp, *dstp;
	int   first_char;
	int   copy;
	int   in_parens;

	new = g_strdup (text);

	first_char = TRUE;
	copy       = TRUE;
	in_parens  = FALSE;
	dstp       = new;

	for (srcp = text; *srcp != '\0'; srcp ++) {
		if (first_char && *srcp == '(') {
			copy       = FALSE;
			first_char = FALSE;
			in_parens  = TRUE;
		}

		if (*srcp == ')')
			in_parens = FALSE;

		if (*srcp == '\n')
			first_char = TRUE;

		if (copy) {
			*dstp = *srcp;
			dstp ++;
		}

		if (*srcp == ':' && ! in_parens && ! first_char && ! copy)
			copy = TRUE;

	}

	*dstp = '\0';

	g_free (text);
	return new;
}

/*
 * Generate a clue out of a range of recent lines in an IM
 * conversation.
 */
static char *
gaim_conv_gen_chunkclue (GaimConversation *conv,
			 int               start_num_lines_back,
			 int               end_num_lines_back,
			 int               relevance)
{
	GaimGtkConversation *gtkconv;
	GtkTextIter  start_iter;
	GtkTextIter  end_iter;
	char        *convchunk;
	int          num_lines;
	char        *clue;

	gtkconv = GAIM_GTK_CONVERSATION (conv);

	num_lines = gtk_text_buffer_get_line_count (GTK_IMHTML (gtkconv->imhtml)->text_buffer);

	start_num_lines_back = NATMIN (num_lines, start_num_lines_back);
	end_num_lines_back   = NATMIN (num_lines, end_num_lines_back);

	gtk_text_buffer_get_end_iter (GTK_IMHTML (gtkconv->imhtml)->text_buffer, &end_iter);
	gtk_text_buffer_get_end_iter (GTK_IMHTML (gtkconv->imhtml)->text_buffer, &start_iter);

	gtk_text_iter_backward_lines (&start_iter, start_num_lines_back);
	gtk_text_iter_backward_lines (&end_iter, end_num_lines_back);

	convchunk = gtk_text_buffer_get_text (GTK_IMHTML (gtkconv->imhtml)->text_buffer,
					      &start_iter, &end_iter, FALSE);

	convchunk = strip_uninteresting (convchunk);

	if (convchunk == NULL || strlen (convchunk) == 0) {
		g_free (convchunk);
		return g_strdup ("");
	}

	clue = dashboard_build_clue (convchunk, "textblock", relevance);

	g_free (convchunk);

	return clue;
}

static char *
gaim_account_get_imname_cluetype (GaimAccount *account)
{
	GaimProtocol proto;

	proto = gaim_account_get_protocol (account);

	switch (proto) {
		case GAIM_PROTO_TOC:      return "aim_name";     break;
		case GAIM_PROTO_OSCAR:    return "aim_name";     break;
		case GAIM_PROTO_YAHOO:    return "yahoo_name";   break;
		case GAIM_PROTO_ICQ:      return "icq_name";     break;
		case GAIM_PROTO_MSN:      return "msn_name";     break;
		case GAIM_PROTO_IRC:      return "irc_name";     break;
		case GAIM_PROTO_JABBER:   return "jabber_name";  break;
		case GAIM_PROTO_NAPSTER:  return "napster_name"; break;
		case GAIM_PROTO_ZEPHYR:   return "zephyr_name";  break;
		case GAIM_PROTO_GADUGADU: return "gg_name";      break;
		case GAIM_PROTO_MOO:      return "moo_name";     break;
		case GAIM_PROTO_TREPIA:   return "trepia_name";  break;

		default:
			break;
	}

	return "";
}

static char *
gaim_conv_gen_buddy_clues (GaimConversation *conv,
			   GaimAccount      *account,
			   const char       *imname)
{
	struct buddy        *b;
	const char          *name        = NULL;
	const char          *proto       = NULL;
	char                *imname_clue;
	char                *name_clue;
	char                *clues;

	g_return_val_if_fail (account != NULL, NULL);
	g_return_val_if_fail (imname != NULL, NULL);

	/* Map the protocol name to a clue type. */
	proto = gaim_account_get_imname_cluetype (account);

	/* Get the buddy alias */
	if (b = gaim_find_buddy (account, imname))
		name = gaim_get_buddy_alias (b);
	if (! strcmp (name, imname))
		name = NULL;

	/* Build the clues */
	imname_clue = dashboard_build_clue (imname, proto,    5);
//	name_clue   = dashboard_build_clue (name,   "name",   5);

	/* Assemble it all */
	clues = g_strdup_printf ("%s", imname_clue);//, name_clue);

	g_free (imname_clue);
//	g_free (name_clue);

	return clues;
}

/*
 * Generates and sends a cluepacket for an IM conversation.
 */
static void
gaim_conv_send_cluepacket (GaimConversation *conv)
{
	GaimGtkConversation *gtkconv;
	GaimAccount         *account;
	const char          *imname;
	char                 context[600];
	char                *cluepacket;

	if (conv == NULL)
		return;

	gtkconv = GAIM_GTK_CONVERSATION (conv);

	fprintf (stderr, "Gaim/Dashboard: Building cluepacket...\n");

	/* Get the imname and name of the buddy we're talking to. */
	account = gaim_conversation_get_account (conv);
	g_return_if_fail (account != NULL);

	imname = gaim_conversation_get_name (conv);
	g_return_if_fail (imname != NULL);

	/* Write the context */
	snprintf (context, sizeof (context), "IM with %s", imname);

	/* Build a cluepacket. */
	cluepacket = dashboard_build_cluepacket_then_free_clues (
		"Gaim",
		gaim_gtkconv_is_focused (gtkconv),
		context,
		gaim_conv_gen_buddy_clues (conv, account, imname),
		gaim_conv_gen_chunkclue   (conv, 20,   0,    10),
//		gaim_conv_gen_chunkclue   (conv, 10,  5,     8),
//		gaim_conv_gen_chunkclue   (conv, 15, 10,     6),
//		gaim_conv_gen_chunkclue   (conv, 20, 15,     4),
		NULL);

	/* Send it off. */
	dashboard_send_raw_cluepacket (cluepacket);

	g_free (cluepacket);
}

void
chat_recv_im (GaimConnection *gc, int id, char **who, char **text)
{
	GaimConversation *c = gaim_find_chat (gc, id);

	fprintf (stderr, "chat_recv_im\n");

	gaim_conv_send_cluepacket (c);

	return;
}

void
chat_sent_im (GaimConnection *gc, int id, char **text)
{
	GaimConversation *c = gaim_find_chat (gc, id);

	fprintf (stderr, "chat_sent_im\n");

	gaim_conv_send_cluepacket (c);

	return;
}

int
im_recv_im (GaimConnection *gc, char **who, char **what, void *m)
{
	GaimConversation *c = gaim_find_conversation (*who);
	GaimGtkConversation *gtkconv;

	gtkconv = GAIM_GTK_CONVERSATION (c);
	fprintf (stderr, "im_recv_im\n");

	gaim_conv_send_cluepacket (c);

	return 0;
}

int
im_sent_im (GaimConnection *gc, char *who, char **what, void *m)
{
	GaimConversation *c = gaim_find_conversation (who);

	fprintf (stderr, "im_sent_im\n");

	gaim_conv_send_cluepacket (c);

	return 0;
}


gboolean
conv_focus_in_cb (GtkWidget     *widget,
		  GdkEventFocus *event,
		  gpointer       user_data)
{
	GaimConversation *conv = user_data;

	if (conv)
		gaim_conv_send_cluepacket (conv);

	return TRUE;
}

void
new_conv (char *who)
{
	GaimConversation *conv = gaim_find_conversation (who);
	GaimGtkConversation *gtkconv;

	fprintf (stderr, "new_conv\n");

	gtkconv = GAIM_GTK_CONVERSATION (conv);

	g_signal_connect (G_OBJECT (gtkconv->entry),
			  "focus_in_event",
			  G_CALLBACK (conv_focus_in_cb),
			  conv);

	g_signal_connect (G_OBJECT (gtkconv->imhtml),
			  "focus_in_event",
			  G_CALLBACK (conv_focus_in_cb),
			  conv);

	gaim_conv_send_cluepacket (conv);
}

void
conv_switch (char *who)
{
	GaimConversation *c = gaim_find_conversation (who);
	GaimGtkConversation *gtkconv;

	fprintf (stderr, "conv_switch\n");

	if (c == NULL)
		return;

	gtkconv = GAIM_GTK_CONVERSATION (c);

	if (gaim_gtkconv_is_focused (gtkconv))
		gaim_conv_send_cluepacket (c);
}

void
chat_join (GaimConnection *gc, int id, char *room)
{
	GaimConversation *c = gaim_find_chat (gc, id);

	fprintf (stderr, "chat_join\n");

	gaim_conv_send_cluepacket (c);
}

static gboolean
plugin_load (GaimPlugin *plugin)
{
	gaim_signal_connect (plugin, event_im_recv,             im_recv_im,   NULL);
	gaim_signal_connect (plugin, event_chat_recv,           chat_recv_im, NULL);
	gaim_signal_connect (plugin, event_im_send,             im_sent_im,   NULL);
	gaim_signal_connect (plugin, event_chat_send,           chat_sent_im, NULL);
	gaim_signal_connect (plugin, event_new_conversation,    new_conv,     NULL);
	gaim_signal_connect (plugin, event_chat_join,           chat_join,    NULL);
	gaim_signal_connect (plugin, event_conversation_switch, conv_switch,  NULL);

	return TRUE;
}

static GaimPluginInfo info =
{
	2,                                                /**< api_version    */
	GAIM_PLUGIN_STANDARD,                             /**< type           */
	GAIM_GTK_PLUGIN_TYPE,                             /**< ui_requirement */
	0,                                                /**< flags          */
	NULL,                                             /**< dependencies   */
	GAIM_PRIORITY_DEFAULT,                            /**< priority       */
	PLUGIN_ID,                                        /**< id             */
	N_ ("Dashboard"),                                 /**< name           */
	VERSION,                                          /**< version        */
	N_ ("FIXME"),                                     /**< summary        */
	N_ ("FIXME."),                                    /**< description    */
	"Nat Friedman <nat@nat.org>",                     /**< author         */
	WEBSITE,                                          /**< homepage       */
	plugin_load,                                      /**< load           */
	NULL,                                             /**< unload         */
	NULL,                                             /**< destroy        */
	NULL,                                             /**< ui_info        */
	NULL                                              /**< extra_info     */
};

static void
init_plugin (GaimPlugin *plugin)
{
}

GAIM_INIT_PLUGIN (dashboard, init_plugin, info);
