/*
 * GNOME Dashboard Text Index
 *
 * stemmer.cs
 *
 * Authors:
 *   Jim Krehl <jimmuhk@yahoo.com>
 *
 * Purpose of this file:
 *
 *   Reduces a word to its stem, e.g. "running" -> "run".
 *
 * TODO:
 *
 *   - replace this with a better stemmer.
 *
 */

/*

   Porter stemmer in Java. The original paper is in

       Porter, 1980, An algorithm for suffix stripping, Program, Vol. 14,
       no. 3, pp 130-137,

   See also http://www.tartarus.org/~martin/PorterStemmer

   History:

   Release 1

   Bug 1 (reported by Gonzalo Parra 16/10/99) fixed as marked below.
   The words 'aed', 'eed', 'oed' leave k at 'a' for step 3, and b[k-1]
   is then out outside the bounds of b.

   Release 2

   Similarly,

   Bug 2 (reported by Steve Dyrdahl 22/2/00) fixed as marked below.
   'ion' by itself leaves j = -1 in the test for 'ion' in step 5, and
   b[j] is then outside the bounds of b.

   Release 3

   Considerably revised 4/9/00 in the light of many helpful suggestions
   from Brian Goetz of Quiotix Corporation (brian@quiotix.com).

   Release 4

   Adapted, modified to c# by jimmyk@ugcs.caltech.edu.

*/

using System;

/**
  * Stemmer, implementing the Porter Stemming Algorithm
  *
  * The Stemmer class transforms a word into its root form.  The input
  * word can be provided a character at time (by calling add()), or at once
  * by calling one of the various stem(something) methods.
  */

namespace Dashboard.Index {

public class StemmingFilter : TokenFilter {
	public override string Filter (string token)
	{
		Stemmer stemmer = new Stemmer();

   		for (int i = 0; i < token.Length; ++i)
			stemmer.Add (Char.ToLower (token [i]));

		stemmer.stem ();

		return stemmer.ToString ();
	}
}

public class Stemmer {
	private char[] b;
	private int i;     /* offset into b */
	private int i_end; /* offset to end of stemmed word */
	private int j;
	private int k;

	private static int INC = 50; /* unit of size whereby b is increased */

	public Stemmer() {
		b = new char[INC];
		i = 0;
		i_end = 0;
	}

   /**
    * Add a character to the word being stemmed.  When you are finished
    * adding characters, you can call stem(void) to stem the word.
    */

	public void Add(char ch) {
		if (i == b.Length) {
			char[] new_b = new char[i+INC];

			for (int c = 0; c < i; c++)
				new_b[c] = b[c];

			b = new_b;
		}

		b[i++] = ch;
	}

   /** Adds wLen characters to the word being stemmed contained in a portion
    * of a char[] array. This is like repeated calls of add(char ch), but
    * faster.
    */

	public void Add(char[] w, int wLen) {
		if (i+wLen >= b.Length) {
			char[] new_b = new char[i+wLen+INC];

			for (int c = 0; c < i; c++)
				new_b[c] = b[c];

			b = new_b;
		}

		for (int c = 0; c < wLen; c++)
			b[i++] = w[c];
	}

   /**
    * After a word has been stemmed, it can be retrieved by toString(),
    * or a reference to the internal buffer can be retrieved by getResultBuffer
    * and getResultLength (which is generally more efficient.)
    */

	public override string ToString() {
		return new string(b,0,i_end);
	}

   /**
    * Returns the length of the word resulting from the stemming process.
    */

	public int GetResultLength() {
		return i_end;
	}

   /**
    * Returns a reference to a character buffer containing the results of
    * the stemming process.  You also need to consult getResultLength()
    * to determine the length of the result.
    */
	public char[] GetResultBuffer() {
		return b;
	}

   /* cons(i) is true <=> b[i] is a consonant. */

	private bool cons(int i) {
		switch (b[i]) {
			case 'a':
			case 'e':
			case 'i':
			case 'o':
			case 'u':
				return false;
			case 'y':
				return i == 0 ? true : !cons(i-1);
		}

		return true;
	}

   /* m() measures the number of consonant sequences between 0 and j. if c is
      a consonant sequence and v a vowel sequence, and <..> indicates arbitrary
      presence,

         <c><v>       gives 0
         <c>vc<v>     gives 1
         <c>vcvc<v>   gives 2
         <c>vcvcvc<v> gives 3
         ....
   */

	private int m() {
		int n = 0;
		int i = 0;

		while (true) {
			if (i > j)
				return n;

			if (!cons(i))
				break;

			i++;
		}

		i++;

		while (true) {
			while(true) {
				if (i > j)
					return n;

				if (cons(i))
					break;

				i++;
			}

			i++;
			n++;

			while(true) {
				if (i > j)
					return n;

				if (!cons(i))
					break;

				i++;
			}

			i++;
		}
	}

   /* vowelinstem() is true <=> 0,...j contains a vowel */

   private bool vowelinstem() {
		int i;

		for (i=0; i<=j; i++)
			if (!cons(i))
				return true;

		return false;
	}

   /* doublec(j) is true <=> j,(j-1) contain a double consonant. */

	private bool doublec(int j) {
		if (j < 1)
			return false;

		if (b[j] != b[j-1])
			return false;

		return cons(j);
	}

   /* cvc(i) is true <=> i-2,i-1,i has the form consonant - vowel - consonant
      and also if the second c is not w,x or y. this is used when trying to
      restore an e at the end of a short word. e.g.

         cav(e), lov(e), hop(e), crim(e), but
         snow, box, tray.

   */

	private bool cvc(int i) {
		if (i < 2 || !cons(i) || cons(i-1) || !cons(i-2))
			return false;

		if (b[i] == 'w' || b[i] == 'x' || b[i] == 'y')
			return false;

		return true;
	}

	private bool ends(string s) {
		int l = s.Length;

		int o = k-l+1;

		if (o < 0)
			return false;

		for (int i=0; i<l; i++)
			if (b[o+i] != s[i])
				return false;

		j = k-l;

		return true;
	}

   /* setto(s) sets (j+1),...k to the characters in the string s, readjusting
      k. */

	private void setto(string s) {
		int l = s.Length;

		int o = j+1;

		for (int i=0; i<l; i++)
			b[o+i] = s[i];

		k = j+l;
	}

   /* r(s) is used further down. */

	private void r(String s) {
		if (m() > 0)
			setto(s);
	}

   /* step1() gets rid of plurals and -ed or -ing. e.g.

          caresses  ->  caress
          ponies    ->  poni
          ties      ->  ti
          caress    ->  caress
          cats      ->  cat

          feed      ->  feed
          agreed    ->  agree
          disabled  ->  disable

          matting   ->  mat
          mating    ->  mate
          meeting   ->  meet
          milling   ->  mill
          messing   ->  mess

          meetings  ->  meet

   */

	private void step1() {
		if (b[k] == 's') {
			if (ends("sses"))
				k -= 2;
			else if (ends("ies"))
				setto("i");
			else if (b[k-1] != 's')
				k--;
		}

		if (ends("eed")) {
			if (m()>0)
				k--;
		}
		else if ((ends("ed") || ends("ing")) && vowelinstem()) {
			k = j;

			if (ends("at"))
				setto("ate");
			else if (ends("bl"))
				setto("ble");
			else if (ends("iz"))
				setto("ize");
			else if (doublec(k)) {
				k--;

				if (b[k] == 'l' || b[k] == 's' || b[k] == 'z')
					k++;
			}
			else if (m() == 1 && cvc(k))
				setto("e");
		}
	}

   /* step2() turns terminal y to i when there is another vowel in the stem. */

	private void step2() {
		if (ends("y") && vowelinstem())
			b[k] = 'i';
	}

   /* step3() maps double suffices to single ones. so -ization ( = -ize plus
      -ation) maps to -ize etc. note that the string before the suffix must give
      m() > 0. */

	private void step3() {
		if (k == 0)
			return; /* For Bug 1 */

		switch (b[k-1]) {
			case 'a':
				if (ends("ational")) {
					r("ate");
					break;
				}

				if (ends("tional")) {
					r("tion");
					break;
				}

				break;

			case 'c':
				if (ends("enci")) {
					r("ence");
					break;
				}

				if (ends("anci")) {
					r("ance");
					break;
				}

				break;

			case 'e':
				if (ends("izer")) {
					r("ize");
					break;
				}

				break;

			case 'l':
				if (ends("bli")) {
					r("ble");
					break;
				}

				if (ends("alli")) {
					r("al");
					break;
				}

				if (ends("entli")) {
					r("ent");
					break;
				}

				if (ends("eli")) {
					r("e");
					break;
				}

				if (ends("ousli")) {
					r("ous");
					break;
				}

				break;

			case 'o':
				if (ends("ization")) {
					r("ize");
					break;
				}

				if (ends("ation")) {
					r("ate");
					break;
				}

				if (ends("ator")) {
					r("ate");
					break;
				}

				break;

			case 's':
				if (ends("alism")) {
					r("al");
					break;
				}

				if (ends("iveness")) {
					r("ive");
					break;
				}

				if (ends("fulness")) {
					r("ful");
					break;
				}

				if (ends("ousness")) {
					r("ous");
					break;
				}

				break;

			case 't':
				if (ends("aliti")) {
					r("al");
					break;
				}

				if (ends("iviti")) {
					r("ive");
					break;
				}

				if (ends("biliti")) {
					r("ble");
					break;
				}

				break;

			case 'g':
				if (ends("logi")) {
					r("log");
					break;
				}
	
				break;
		}
	}

   /* step4() deals with -ic-, -full, -ness etc. similar strategy to step3. */

	private void step4() {
		switch (b[k]) {
			case 'e':
				if (ends("icate")) {
					r("ic");
					break;
				}

				if (ends("ative")) {
					r("");
					break;
				}

				if (ends("alize")) {
					r("al");
					break;
				}

				break;

			case 'i':
				if (ends("iciti")) {
					r("ic");
					break;
				}

				break;

			case 'l':
				if (ends("ical")) {
					r("ic");
					break;
				}

				if (ends("ful")) {
					r("");
					break;
				}

				break;

			case 's':
				if (ends("ness")) {
					r("");
					break;
				}

				break;
		}
	}

   /* step5() takes off -ant, -ence etc., in context <c>vcvc<v>. */

	private void step5 () {
		if (k == 0)
			return; /* for Bug 1 */

		switch (b [k-1]) {
		case 'a':
			if (ends ("al"))
				goto bad_mother_fucker;

			return;

		case 'c':
			if (ends ("ance"))
				goto bad_mother_fucker;

			if (ends ("ence"))
				goto bad_mother_fucker;
				
			return;

		case 'e':
			if (ends ("er"))
				goto bad_mother_fucker;

			return;

		case 'i':
			if (ends ("ic"))
				goto bad_mother_fucker;

			return;

		case 'l':
			if (ends ("able"))
				goto bad_mother_fucker;

			if (ends ("ible"))
				goto bad_mother_fucker;

			return;

		case 'n':
			if (ends ("ant"))
				goto bad_mother_fucker;

			if (ends ("ement"))
				goto bad_mother_fucker;

			if (ends ("ment"))
				goto bad_mother_fucker;

			/* element etc. not stripped before the m */

			if (ends ("ent"))
				goto bad_mother_fucker;

			return;

		case 'o':
			if (ends ("ion") && j >= 0 && (b [j] == 's' || b [j] == 't'))
				goto bad_mother_fucker;

			/* j >= 0 fixes Bug 2 */

			if (ends ("ou"))
				goto bad_mother_fucker;

			return;

			/* takes care of -ous */

		case 's':
			if (ends ("ism"))
				goto bad_mother_fucker;

			return;

		case 't':
			if (ends ("ate"))
				goto bad_mother_fucker;

			if (ends ("iti"))
				goto bad_mother_fucker;

			return;

		case 'u':
			if (ends ("ous"))
				goto bad_mother_fucker;
			
			return;

		case 'v':
			if (ends ("ive"))
				goto bad_mother_fucker;

			return;

		case 'z':
			if (ends ("ize"))
				goto bad_mother_fucker;
			
			return;

		default:
			return;
		}

		bad_mother_fucker:

		if (m () > 1)
			k = j;
	}

   /* step6() removes a final -e if m() > 1. */

	private void step6() {
		j = k;

		if (b[k] == 'e') {
			int a = m();

			if (a > 1 || a == 1 && !cvc(k-1))
				k--;
		}

		if (b[k] == 'l' && doublec(k) && m() > 1)
			k--;
	}

   /** Stem the word placed into the Stemmer buffer through calls to add().
    * Returns true if the stemming process resulted in a word different
    * from the input.  You can retrieve the result with
    * getResultLength()/getResultBuffer() or toString().
    */

	public void stem() {
		k = i - 1;

		if (k > 1) {
			step1();
			step2();
			step3();
			step4();
			step5();
			step6();
		}

		i_end = k+1;
		i = 0;
	}
}

}
